@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (c) 2008, 2010, 2011, 2012, 2013, 2014 The University of Utah
+// Copyright (c) 2008, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017 The University of Utah
 // All rights reserved.
 //
 // This file is part of `csmith', a random generator of C programs.
@@ -29,10 +29,14 @@
 
 ///////////////////////////////////////////////////////////////////////////////
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "CGOptions.h"
 #include <iostream>
-#include <assert.h>
-#include <string.h>
+#include <cassert>
+#include <cstring>
 #include <map>
 #include "Fact.h"
 #include "DefaultOutputMgr.h"
@@ -136,6 +140,7 @@ DEFINE_GETTER_SETTER_BOOL(longlong)
 DEFINE_GETTER_SETTER_BOOL(int8)
 DEFINE_GETTER_SETTER_BOOL(uint8)
 DEFINE_GETTER_SETTER_BOOL(enable_float)
+DEFINE_GETTER_SETTER_BOOL(strict_float)
 DEFINE_GETTER_SETTER_BOOL(pointers)
 DEFINE_GETTER_SETTER_BOOL(arrays)
 DEFINE_GETTER_SETTER_BOOL(strict_const_arrays)
@@ -147,6 +152,7 @@ DEFINE_GETTER_SETTER_BOOL(arg_unions)
 DEFINE_GETTER_SETTER_BOOL(volatiles)
 DEFINE_GETTER_SETTER_BOOL(volatile_pointers)
 DEFINE_GETTER_SETTER_BOOL(const_pointers)
+DEFINE_GETTER_SETTER_BOOL(global_variables)
 DEFINE_GETTER_SETTER_BOOL(access_once)
 DEFINE_GETTER_SETTER_BOOL(strict_volatile_rule)
 DEFINE_GETTER_SETTER_BOOL(addr_taken_of_locals)
@@ -158,8 +164,7 @@ DEFINE_GETTER_SETTER_BOOL(divs)
 DEFINE_GETTER_SETTER_BOOL(muls)
 DEFINE_GETTER_SETTER_BOOL(accept_argc)
 DEFINE_GETTER_SETTER_BOOL(random_random)
-DEFINE_GETTER_SETTER_INT(stop_by_stmt) 
-DEFINE_GETTER_SETTER_BOOL(deputy) 
+DEFINE_GETTER_SETTER_INT(stop_by_stmt)
 DEFINE_GETTER_SETTER_BOOL(step_hash_by_stmt)
 DEFINE_GETTER_SETTER_BOOL(compound_assignment)
 DEFINE_GETTER_SETTER_STRING_REF(dump_default_probabilities)
@@ -190,7 +195,10 @@ DEFINE_GETTER_SETTER_BOOL(use_embedded_assigns);
 DEFINE_GETTER_SETTER_BOOL(use_comma_exprs);
 DEFINE_GETTER_SETTER_BOOL(take_union_field_addr);
 DEFINE_GETTER_SETTER_BOOL(vol_struct_union_fields);
+DEFINE_GETTER_SETTER_BOOL(const_struct_union_fields);
 DEFINE_GETTER_SETTER_BOOL(lang_cpp);
+DEFINE_GETTER_SETTER_BOOL(cpp11);
+DEFINE_GETTER_SETTER_BOOL(fast_execution);
 
 void
 CGOptions::set_default_builtin_kinds()
@@ -253,6 +261,7 @@ CGOptions::set_default_settings(void)
 	int8(true);
 	uint8(true);
 	enable_float(false);
+	strict_float(false);
 	pointers(true);
 	arrays(true);
 	strict_const_arrays(false);
@@ -264,13 +273,13 @@ CGOptions::set_default_settings(void)
 	volatiles(true);
 	volatile_pointers(true);
 	const_pointers(true);
+	global_variables(true);
 	consts(true);
 	dangling_global_ptrs(true);
 	divs(true);
 	muls(true);
 	accept_argc(true);
 	stop_by_stmt(-1);
-	deputy(false);
 	step_hash_by_stmt(false);
 	const_as_condition(false);
 	match_exact_qualifiers(false);
@@ -297,12 +306,24 @@ CGOptions::set_default_settings(void)
 	use_comma_exprs(true);
 	take_union_field_addr(true);
 	vol_struct_union_fields(true);
+	const_struct_union_fields(true);
 	addr_taken_of_locals(true);
 	lang_cpp(false);
+	cpp11(false);
+  fast_execution(false);
 
 	set_default_builtin_kinds();
 }
-	
+
+// Add options necessary for cpp 
+void
+CGOptions::fix_options_for_cpp(void)
+{
+	match_exact_qualifiers(true);
+    vol_struct_union_fields(false);		// makes implementation of volatile structs easier
+    const_struct_union_fields(false);	// restriction of current implementation; TODO
+}
+
 /*
    looking for the platform info file in the working directory
    and load platform specific information. If not found, use
@@ -310,7 +331,7 @@ CGOptions::set_default_settings(void)
    to the file
 */
 void
-CGOptions::set_platform_specific_options(void) 
+CGOptions::set_platform_specific_options(void)
 {
 	const char* int_str = "integer size = ";
 	const char* ptr_str = "pointer size = ";
@@ -331,12 +352,12 @@ CGOptions::set_platform_specific_options(void)
 				string s = line.substr(strlen(int_str));
 				StringUtils::chop(s);
 				int_size(StringUtils::str2int(s));
-			} 
+			}
 			if (line.substr(0, strlen(ptr_str)) == ptr_str) {
 				string s = line.substr(strlen(ptr_str));
 				StringUtils::chop(s);
 				pointer_size(StringUtils::str2int(s));
-			} 
+			}
 		}
 		if (!int_size_) {
 			cout << "please specify integer size in " << PLATFORM_CONFIG_FILE << endl;
@@ -347,17 +368,17 @@ CGOptions::set_platform_specific_options(void)
 			exit(-1);
 		}
 		conf.close();
-	}  
+	}
 }
 
-int 
-CGOptions::int_size(void) 
+int
+CGOptions::int_size(void)
 {
 	Bookkeeper::rely_on_int_size = true;
 	return int_size_;
-} 
+}
 
-int 
+int
 CGOptions::pointer_size(void)
 {
 	return pointer_size_;
@@ -428,7 +449,7 @@ bool CGOptions::resolve_exhaustive_options()
 		conflict_msg_ = "expand-struct cannot be used with --no-struct";
 		return true;
 	}
-	
+
 	if (CGOptions::has_extension_support()) {
 		conflict_msg_ = "exhaustive mode doesn't support splat|klee|crest|coverage-test extension";
 		return true;
@@ -475,7 +496,7 @@ CGOptions::has_extension_conflict()
 bool
 CGOptions::has_extension_support()
 {
-	return (CGOptions::splat() || CGOptions::klee() 
+	return (CGOptions::splat() || CGOptions::klee()
 		|| CGOptions::crest() || CGOptions::coverage_test());
 }
 
@@ -483,7 +504,7 @@ bool
 CGOptions::allow_int64()
 {
 	return ((!CGOptions::has_extension_support()) &&
-			CGOptions::math64() && 
+			CGOptions::math64() &&
 			CGOptions::longlong());
 }
 
@@ -537,7 +558,7 @@ CGOptions::has_conflict(void)
 			return true;
 		}
 	}
-	
+
 	if (!CGOptions::delta_monitor().empty()) {
 		string msg;
 		if (!DeltaMonitor::init(msg, CGOptions::delta_monitor(), CGOptions::delta_output())) {
@@ -548,13 +569,18 @@ CGOptions::has_conflict(void)
 
 	if (!CGOptions::go_delta().empty()) {
 		string msg;
-		if (!DeltaMonitor::init_for_running(msg, CGOptions::go_delta(), CGOptions::delta_output(), 
+		if (!DeltaMonitor::init_for_running(msg, CGOptions::go_delta(), CGOptions::delta_output(),
 				CGOptions::delta_input(), CGOptions::no_delta_reduction())) {
 			conflict_msg_ = msg;
 			return true;
 		}
 	}
 
+	if (!CGOptions::lang_cpp() && CGOptions::cpp11()) {
+		conflict_msg_ = "--cpp11 option makes sense only with --lang-cpp option enabled.";
+		return true;
+	}
+
 #if 0
 	Probabilities *prob = Probabilities::GetInstance();
 	if (!(CGOptions::dump_default_probabilities().empty()))
@@ -578,15 +604,15 @@ CGOptions::monitored_funcs(std::string fnames)
 	parse_string_options(fnames, OutputMgr::monitored_funcs_);
 }
 
-void 
+void
 CGOptions::parse_string_options(string vname, vector<std::string> &v)
 {
 	if (vname.empty()) return;
-	// parse the string if there are multiple variable names in format "v1,v2,..." 
+	// parse the string if there are multiple variable names in format "v1,v2,..."
 	size_t pos1 = 0;
 	size_t pos2 = 0;
 	do {
-		pos2 = vname.find_first_of(',', pos1); 
+		pos2 = vname.find_first_of(',', pos1);
 		string var_name = (pos2 != string::npos) ? vname.substr(pos1, pos2 - pos1) : vname.substr(pos1);
 		v.push_back(var_name);
 		pos1 = pos2 + 1;
@@ -605,19 +631,19 @@ CGOptions::x86_64()
 
 std::string CGOptions::vol_tests_mach_ = "";
 
-std::string 
+std::string
 CGOptions::vol_tests_mach(void)
 {
 	return CGOptions::vol_tests_mach_;
 }
 
-void 
+void
 CGOptions::safe_math_wrapper(string ids)
 {
 	StringUtils::split_int_string(ids, safe_math_wrapper_ids_, ",");
 }
 
-bool 
+bool
 CGOptions::safe_math_wrapper(int id)
 {
 	// if no safe math wrapper ids specified, assume all needs wrapper
