@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (c) 2007, 2008, 2009, 2010, 2011, 2013 The University of Utah
+// Copyright (c) 2007, 2008, 2009, 2010, 2011, 2013, 2015, 2016, 2017 The University of Utah
 // All rights reserved.
 //
 // This file is part of `csmith', a random generator of C programs.
@@ -27,7 +27,11 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 
-#include <assert.h>
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include <cassert>
 #include <iostream>
 #include "CVQualifiers.h"
 #include "Type.h"
@@ -81,7 +85,7 @@ CVQualifiers::~CVQualifiers()
 
 CVQualifiers &
 CVQualifiers::operator=(const CVQualifiers &qfer)
-{	
+{
 	if (this == &qfer) {
 		return *this;
 	}
@@ -101,25 +105,25 @@ CVQualifiers::operator=(const CVQualifiers &qfer)
   *    const is NOT more qualified than volatile
   *    ...
   *  notice "const int**" is not convertable from "int**"
-  *  as explained in 
+  *  as explained in
   * http://www.embedded.com/columns/programmingpointers/180205632?_requestid=488055
   **************************************************************/
-bool 
+bool
 CVQualifiers::stricter_than(const CVQualifiers& qfer) const
 {
-	size_t i; 
+	size_t i;
 	assert(is_consts.size() == is_volatiles.size());
 	const vector<bool>& v_consts = qfer.get_consts();
 	const vector<bool>& v_volatiles = qfer.get_volatiles();
 	if (is_consts.size() != v_consts.size() || is_volatiles.size() != v_volatiles.size()) {
 		return false;
 	}
-	
+
 	size_t depth = is_consts.size();
-	// check "const" qualifier first 
+	// check "const" qualifier first
 	for (i=0; i<depth; i++) {
 		// for special rule: "const int**" is not convertable from "int**"
-		// actually for a level that is followed by two "*"s, we have to match 
+		// actually for a level that is followed by two "*"s, we have to match
 		// "const" qualifier
 		if (depth - i > 2 && is_consts[i] != v_consts[i]) {
 			return false;
@@ -144,11 +148,11 @@ CVQualifiers::stricter_than(const CVQualifiers& qfer) const
 		if (v_volatiles[i] && !is_volatiles[i]) {
 			return false;
 		}
-	}		
+	}
 	return true;
 }
 
-bool 
+bool
 CVQualifiers::match(const CVQualifiers& qfer) const
 {
 	if (wildcard) {
@@ -165,7 +169,7 @@ CVQualifiers::match(const CVQualifiers& qfer) const
 	return (!accept_stricter && stricter_than(qfer)) || (accept_stricter && qfer.stricter_than(*this));
 }
 
-bool 
+bool
 CVQualifiers::match_indirect(const CVQualifiers& qfer) const
 {
 	if (wildcard) {
@@ -187,7 +191,7 @@ CVQualifiers::match_indirect(const CVQualifiers& qfer) const
 void
 CVQualifiers::make_scalar_volatiles(std::vector<bool> &volatiles)
 {
-	if (!CGOptions::volatile_pointers()) {
+	if (!CGOptions::volatile_pointers() || !CGOptions::global_variables()) {
 		for (size_t i=1; i<volatiles.size(); i++)
 			volatiles[i] = false;
 	}
@@ -205,10 +209,10 @@ CVQualifiers::make_scalar_consts(std::vector<bool> &consts)
 	}
 }
 
-/* 
+/*
  * generate a random CV qualifier vector that is looser or stricter than this one
  */
-CVQualifiers 
+CVQualifiers
 CVQualifiers::random_qualifiers(bool no_volatile, Effect::Access access, const CGContext &cg_context) const
 {
 	std::vector<bool> volatiles;
@@ -241,10 +245,10 @@ CVQualifiers::random_qualifiers(bool no_volatile, Effect::Access access, const C
 	return CVQualifiers(consts, volatiles);
 }
 
-/* 
+/*
  * generate a random CV qualifier vector that is looser than this one
  */
-CVQualifiers 
+CVQualifiers
 CVQualifiers::random_loose_qualifiers(bool no_volatile, Effect::Access access, const CGContext &cg_context) const
 {
 	std::vector<bool> volatiles;
@@ -278,16 +282,38 @@ CVQualifiers::random_loose_qualifiers(bool no_volatile, Effect::Access access, c
 }
 
 CVQualifiers
-CVQualifiers::random_qualifiers(const Type* t, Effect::Access access, 
+CVQualifiers::random_qualifiers(const Type* t, Effect::Access access,
 				const CGContext &cg_context, bool no_volatile)
 {
 	return random_qualifiers(t, access, cg_context, no_volatile, RegularConstProb, RegularVolatileProb);
 }
 
-CVQualifiers 
-CVQualifiers::random_qualifiers(const Type* t, Effect::Access access, const CGContext &cg_context, bool no_volatile, 
+static bool is_volatile_ok_on_one_level(const Type* t)
+{
+	if (!CGOptions::lang_cpp()) return true;
+	if (t->eType != eStruct && t->eType != eUnion) return true;
+
+	if (!t->has_assign_ops()) return false;
+	if (t->eType == eStruct) return true;
+
+	// Union with a struct field: we can't make it volatile, or we won't be able to assign to/from that field
+	for (size_t i = 0; i < t->fields.size(); ++i) {
+		const Type* field = t->fields[i];
+
+		if (field->eType == eStruct)
+			return false;
+		if (field->eType == eUnion){
+			if (!is_volatile_ok_on_one_level(field))
+				return false;
+		}
+	}
+	return true;
+}
+
+CVQualifiers
+CVQualifiers::random_qualifiers(const Type* t, Effect::Access access, const CGContext &cg_context, bool no_volatile,
 					unsigned int const_prob, unsigned int volatile_prob)
-{ 
+{
 	CVQualifiers ret_qfer;
 	if (t==0) {
 		return ret_qfer;
@@ -295,48 +321,40 @@ CVQualifiers::random_qualifiers(const Type* t, Effect::Access access, const CGCo
 	bool isVolatile = false;
 	bool isConst = false;
 	std::vector<bool> is_consts, is_volatiles;
-	const Effect &effect_context = cg_context.get_effect_context(); 
+	const Effect &effect_context = cg_context.get_effect_context();
 
 	// set random volatile/const properties for each level of indirection for pointers
+	// First set up the vectors with correct number of qualifiers:
+	unsigned level = 0;
 	const Type* tmp = t->ptr_type;
+	while (tmp) {
+		++level;
+		is_consts.push_back(false);
+		is_volatiles.push_back(false);
+		tmp = tmp->ptr_type;
+	}
+	// Then make the random properties (properties need to be in reverse order): 
+	tmp = t->ptr_type;
 	while (tmp) {   
-		DEPTH_GUARD_BY_DEPTH_RETURN(2, ret_qfer);
-		isVolatile = rnd_flipcoin(volatile_prob);
-		ERROR_GUARD(ret_qfer);
-		isConst = rnd_flipcoin(const_prob); 
-		ERROR_GUARD(ret_qfer);
+		bool volatile_ok = is_volatile_ok_on_one_level(tmp);
+		isVolatile = volatile_ok? rnd_flipcoin(volatile_prob): false;
+		isConst = rnd_flipcoin(const_prob);
 		if (isVolatile && isConst && !CGOptions::allow_const_volatile()) {
 			isConst = false;
 		}
-		is_consts.push_back(isConst);
-		is_volatiles.push_back(isVolatile);
+		assert(level > 0);
+		is_consts[level-1] = isConst;
+		is_volatiles[level-1] = isVolatile;
+		--level;
 		tmp = tmp->ptr_type;
 	}
+
 	// set random volatile/const properties for variable itself
-	bool volatile_ok = effect_context.is_side_effect_free();
+	bool volatile_ok = effect_context.is_side_effect_free() && is_volatile_ok_on_one_level(t);
 	bool const_ok = (access != Effect::WRITE);
 
-	isVolatile = false;
-	isConst = false;
-
-	if (volatile_ok && const_ok) {
-		DEPTH_GUARD_BY_DEPTH_RETURN(2, ret_qfer);
-		isVolatile = rnd_flipcoin(volatile_prob);
-		ERROR_GUARD(ret_qfer);
-		isConst = rnd_flipcoin(const_prob);
-		ERROR_GUARD(ret_qfer);  
-	}
-	else if (volatile_ok) {
-		DEPTH_GUARD_BY_DEPTH_RETURN(1, ret_qfer);
-		isVolatile = rnd_flipcoin(volatile_prob);
-		ERROR_GUARD(ret_qfer);
-	}
-	else if (const_ok) {
-		DEPTH_GUARD_BY_DEPTH_RETURN(1, ret_qfer);
-		isConst = rnd_flipcoin(const_prob);
-		ERROR_GUARD(ret_qfer);
-	}
-
+	isVolatile = volatile_ok ? rnd_flipcoin(volatile_prob) : false;
+	isConst = const_ok ? rnd_flipcoin(const_prob) : false;
 	if (isVolatile && isConst && !CGOptions::allow_const_volatile()) {
 		isConst = false;
 	}
@@ -353,27 +371,27 @@ CVQualifiers::random_qualifiers(const Type* t, Effect::Access access, const CGCo
 	return CVQualifiers(is_consts, is_volatiles);
 }
 
-/* 
+/*
  * make a random qualifier for type t, assuming non context,
  * and no volatile allowed
  */
-CVQualifiers 
+CVQualifiers
 CVQualifiers::random_qualifiers(const Type* t)
-{ 
+{
 	return random_qualifiers(t, Effect::READ, CGContext::get_empty_context(), true);
 }
 
-/* 
- * be careful to use it because it will generate volatile without knowing the context. 
+/*
+ * be careful to use it because it will generate volatile without knowing the context.
  * Only used to generate qulifiers for struct/unions
  */
-CVQualifiers 
+CVQualifiers
 CVQualifiers::random_qualifiers(const Type* t, unsigned int const_prob, unsigned int volatile_prob)
-{ 
+{
 	return random_qualifiers(t, Effect::READ, CGContext::get_empty_context(), false, const_prob, volatile_prob);
 }
 
-vector<bool> 
+vector<bool>
 CVQualifiers::random_stricter_consts(void) const
 {
 	vector<bool> consts;
@@ -401,7 +419,7 @@ CVQualifiers::random_stricter_consts(void) const
 	return consts;
 }
 
-vector<bool> 
+vector<bool>
 CVQualifiers::random_stricter_volatiles(void) const
 {
 	vector<bool> volatiles;
@@ -428,7 +446,7 @@ CVQualifiers::random_stricter_volatiles(void) const
 	return volatiles;
 }
 
-vector<bool> 
+vector<bool>
 CVQualifiers::random_looser_consts(void) const
 {
 	vector<bool> consts;
@@ -450,7 +468,7 @@ CVQualifiers::random_looser_consts(void) const
 	return consts;
 }
 
-vector<bool> 
+vector<bool>
 CVQualifiers::random_looser_volatiles(void) const
 {
 	vector<bool> volatiles;
@@ -472,7 +490,7 @@ CVQualifiers::random_looser_volatiles(void) const
 	return volatiles;
 }
 
-void 
+void
 CVQualifiers::add_qualifiers(bool is_const, bool is_volatile)
 {
 	is_consts.push_back(is_const);
@@ -481,7 +499,7 @@ CVQualifiers::add_qualifiers(bool is_const, bool is_volatile)
 
 
 // actually add qualifiers to pointers
-CVQualifiers 
+CVQualifiers
 CVQualifiers::random_add_qualifiers(bool no_volatile) const
 {
 	CVQualifiers qfer = *this;
@@ -503,7 +521,7 @@ CVQualifiers::random_add_qualifiers(bool no_volatile) const
 	else
 		is_const = rnd_flipcoin(RegularConstProb);
 	ERROR_GUARD(qfer);
-	//bool is_volatile = no_volatile ? false : rnd_upto(RegularVolatileProb);  
+	//bool is_volatile = no_volatile ? false : rnd_upto(RegularVolatileProb);
 	bool is_volatile;
 	if (no_volatile || !CGOptions::volatile_pointers())
 		is_volatile = false;
@@ -515,7 +533,7 @@ CVQualifiers::random_add_qualifiers(bool no_volatile) const
 	return qfer;
 }
 
-void 
+void
 CVQualifiers::remove_qualifiers(int len)
 {
 	int i;
@@ -525,7 +543,7 @@ CVQualifiers::remove_qualifiers(int len)
 	}
 }
 
-CVQualifiers 
+CVQualifiers
 CVQualifiers::indirect_qualifiers(int level) const
 {
 	if (level == 0 || wildcard) {
@@ -549,7 +567,7 @@ CVQualifiers::indirect_qualifiers(int level) const
 /*
  * check if the indirect depth of type matches qualifier size
  */
-bool 
+bool
 CVQualifiers::sanity_check(const Type* t) const
 {
 	assert(t);
@@ -588,38 +606,8 @@ CVQualifiers::output_qualified_type(const Type* t, std::ostream &out) const
 	}
 }
 
-void
-CVQualifiers::output_qualified_type_with_deputy_annotation(const Type* t, std::ostream &out, const vector<string>& annotations) const
-{
-	assert(t);
-	assert(sanity_check(t));
-	assert(is_consts.size() == annotations.size()+1);
-	size_t i;
-	const Type* base = t->get_base_type();
-	for (i=0; i<is_consts.size(); i++) {
-		if (i>0) {
-			out << "* ";
-			out << annotations[i-1] << " ";
-		}
-		if (is_consts[i]) {
-			if (!CGOptions::consts())
-				assert(0);
-			out << "const ";
-		}
-		if (is_volatiles[i]) {
-			if (!CGOptions::volatiles())
-				assert(0);
-			out << "volatile ";
-		}
-		if (i==0) {
-			base->Output(out);
-			out << " ";
-		}
-	}
-}
-
-bool 
-CVQualifiers::is_const_after_deref(int deref_level) const 
+bool
+CVQualifiers::is_const_after_deref(int deref_level) const
 {
 	if (deref_level < 0) {
 		return false;
@@ -628,9 +616,9 @@ CVQualifiers::is_const_after_deref(int deref_level) const
 	assert(len > static_cast<size_t>(deref_level));
 	return is_consts[len - deref_level - 1];
 }
-	
-bool 
-CVQualifiers::is_volatile_after_deref(int deref_level) const 
+
+bool
+CVQualifiers::is_volatile_after_deref(int deref_level) const
 {
 	if (deref_level < 0) {
 		return false;
@@ -646,7 +634,7 @@ CVQualifiers::is_volatile_after_deref(int deref_level) const
 	return is_volatiles[len - deref_level - 1];
 }
 
-void 
+void
 CVQualifiers::set_const(bool is_const, int pos)
 {
 	int len = is_consts.size();
@@ -655,7 +643,7 @@ CVQualifiers::set_const(bool is_const, int pos)
 	}
 }
 
-void 
+void
 CVQualifiers::set_volatile(bool is_volatile, int pos)
 {
 	int len = is_volatiles.size();
@@ -674,10 +662,10 @@ CVQualifiers::restrict(Effect::Access access, const CGContext& cg_context)
 		set_volatile(false);
 	}
 }
-	
+
 /*
- * For now, only used to generate all qualifiers for struct fields. 
- * Also, since we don't support fields with pointer types, we only 
+ * For now, only used to generate all qualifiers for struct fields.
+ * Also, since we don't support fields with pointer types, we only
  * enumerate the first level of qualifiers.
  */
 void
@@ -688,8 +676,8 @@ CVQualifiers::get_all_qualifiers(vector<CVQualifiers> &quals, unsigned int const
 	qual_enumerator.add_bool_elem("volatile_prob", volatile_prob);
 	Enumerator<string> *i;
 	for (i = qual_enumerator.begin(); i != qual_enumerator.end(); i = i->next()) {
-		bool isConst = i->get_elem("const_prob");
-		bool isVolatile = i->get_elem("volatile_prob");
+		bool isConst = i->get_elem("const_prob") != 0;
+		bool isVolatile = i->get_elem("volatile_prob") != 0;
 
 		vector<bool> consts;
 		vector<bool> volatiles;
@@ -718,7 +706,7 @@ CVQualifiers::OutputFirstQuals(std::ostream &out) const
 	}
 }
 
-void 
+void
 CVQualifiers::output() const
 {
 	size_t i;
