@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (c) 2007, 2008, 2009, 2010, 2011 The University of Utah
+// Copyright (c) 2007, 2008, 2009, 2010, 2011, 2015, 2017 The University of Utah
 // All rights reserved.
 //
 // This file is part of `csmith', a random generator of C programs.
@@ -26,11 +26,16 @@
 // CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
-#ifdef WIN32 
+
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#ifdef WIN32
 #pragma warning(disable : 4786)   /* Disable annoying warning messages */
 #endif
 
-#include "FactMgr.h" 
+#include "FactMgr.h"
 
 #include <cassert>
 #include <sstream>
@@ -59,8 +64,8 @@
 #include "Lhs.h"
 #include "CFGEdge.h"
 
-using namespace std; 
- 
+using namespace std;
+
 std::vector<Fact*> FactMgr::meta_facts;
 
 void
@@ -70,25 +75,25 @@ FactMgr::add_new_var_fact_and_update_inout_maps(const Block* blk, const Variable
 		assert(var->is_global());
 	}
 	// var is global if blk == 0, we add fact to all blocks of this function
-	// otherwise, only add fact to blocks equal or below blk (variable is declared in blk)  
-	for (size_t i=0; i<FactMgr::meta_facts.size(); i++) { 
-		FactVec facts = FactMgr::meta_facts[i]->abstract_fact_for_var_init(var); 
+	// otherwise, only add fact to blocks equal or below blk (variable is declared in blk)
+	for (size_t i=0; i<FactMgr::meta_facts.size(); i++) {
+		FactVec facts = FactMgr::meta_facts[i]->abstract_fact_for_var_init(var);
 		for (size_t k=0; k<facts.size(); k++) {
 			const Fact* f = facts[k];
 			// sometimes the facts is already included during the creation of child
 			// functions. so need this check
 			if (find_related_fact(global_facts, f) == 0) {
 				global_facts.push_back(f);
-			} 
+			}
 
 			map<const Statement*, FactVec>::iterator iter;
-			for(iter = map_facts_in.begin(); iter != map_facts_in.end(); ++iter) {  
+			for(iter = map_facts_in.begin(); iter != map_facts_in.end(); ++iter) {
 				const Statement* stm = iter->first;
 				if (stm && (stm->in_block(blk) || blk == NULL)) {
 					iter->second.push_back(f);
 				}
 			}
-			for(iter = map_facts_out.begin(); iter != map_facts_out.end(); ++iter) {  
+			for(iter = map_facts_out.begin(); iter != map_facts_out.end(); ++iter) {
 				const Statement* stm = iter->first;
 				assert(stm);
 				if (blk) {
@@ -96,28 +101,28 @@ FactMgr::add_new_var_fact_and_update_inout_maps(const Block* blk, const Variable
 				} else {
 					iter->second.push_back(f);
 				}
-			} 
+			}
 		}
 	}
 }
 
-void 
+void
 FactMgr::add_param_facts(const vector<const Expression*>& param_values, FactVec& facts)
-{ 
+{
 	for (size_t i=0; i<func->param.size(); i++) {
 		const Variable* var = func->param[i];
-		const Expression* value = param_values[i]; 
-		Lhs lhs(*var); 
+		const Expression* value = param_values[i];
+		Lhs lhs(*var);
 		FactMgr::update_fact_for_assign(&lhs, value, facts);
 	}
 }
 
-void 
+void
 FactMgr::add_new_var_fact(const Variable* v, FactVec& facts)
 {
 	assert(v);
-	for (size_t i=0; i<FactMgr::meta_facts.size(); i++) { 
-		FactVec tmp_facts = FactMgr::meta_facts[i]->abstract_fact_for_var_init(v);   
+	for (size_t i=0; i<FactMgr::meta_facts.size(); i++) {
+		FactVec tmp_facts = FactMgr::meta_facts[i]->abstract_fact_for_var_init(v);
 		for (size_t k=0; k<tmp_facts.size(); k++) {
 			const Fact* f = tmp_facts[k];
 			// sometimes the facts is already included during the creation of child
@@ -130,7 +135,7 @@ FactMgr::add_new_var_fact(const Variable* v, FactVec& facts)
 }
 
 /* update facts env for out-of-scope variables */
-void 
+void
 FactMgr::update_facts_for_oos_vars(const vector<Variable*>& vs, FactVec& facts)
 {
 	vector<const Variable*> vars;
@@ -138,7 +143,7 @@ FactMgr::update_facts_for_oos_vars(const vector<Variable*>& vs, FactVec& facts)
 	return update_facts_for_oos_vars(vars, facts);
 }
 
-void 
+void
 FactMgr::update_facts_for_oos_vars(const vector<const Variable*>& vars, FactVec& facts)
 {
 	//print_facts(facts);
@@ -177,7 +182,7 @@ FactMgr::update_facts_for_oos_vars(const vector<const Variable*>& vars, FactVec&
  * remove facts concerning variables local to a function
  * hint: relevant facts are those concerns variable visible at the end of this function
  */
-void 
+void
 FactMgr::remove_function_local_facts(std::vector<const Fact*>& inputs, const Statement* stm)
 {
 	size_t i;
@@ -189,14 +194,14 @@ FactMgr::remove_function_local_facts(std::vector<const Fact*>& inputs, const Sta
 		const Variable* v = inputs[i]->get_var();
 		// if it's fact for a local variable of this function, or return variable
 		// of another function, we are not interested in them after exit function
-		if (func->is_var_on_stack(v, stm) || (v->is_rv() && !func->rv->match(v))) { 
+		if (func->is_var_on_stack(v, stm) || (v->is_rv() && !func->rv->match(v))) {
 			inputs.erase(inputs.begin() + i);
 			i--;
-			len--; 
+			len--;
 		}
 	}
-	// mark any remaining facts that may point to a local vars of this 
-	// function as "point to garbage"  
+	// mark any remaining facts that may point to a local vars of this
+	// function as "point to garbage"
 	for (i=0; i<inputs.size(); i++) {
 		if (inputs[i]->eCat == ePointTo) {
 			FactPointTo* f = (FactPointTo*)(inputs[i]);
@@ -212,52 +217,52 @@ void
 FactMgr::setup_in_out_maps(bool first_time)
 {
 	if (first_time) {
-		// first time revisit, create map_facts_in_final and map_facts_out_final with cloned facts 
+		// first time revisit, create map_facts_in_final and map_facts_out_final with cloned facts
 		map<const Statement*, vector<const Fact*> >::const_iterator iter;
 		for(iter = map_facts_in.begin(); iter != map_facts_in.end(); ++iter) {
 			const Statement* stm = iter->first;
 			const vector<const Fact*>& facts1 = iter->second;
 			map_facts_in_final[stm] = copy_facts(facts1);
-		}    
+		}
 		for(iter = map_facts_out.begin(); iter != map_facts_out.end(); ++iter) {
 			const Statement* stm = iter->first;
-			const vector<const Fact*>& facts1 = iter->second; 
+			const vector<const Fact*>& facts1 = iter->second;
 			map_facts_out_final[stm] = copy_facts(facts1);
-		}    
+		}
 	}
-	else {  
+	else {
 		// not the 1st time revisit
 		// combine facts_in and facts_out from this invocation with facts from previous invocations
 		map<const Statement*, vector<Fact*> >::iterator iter;
 		for(iter = map_facts_in_final.begin(); iter != map_facts_in_final.end(); ++iter) {
 			const Statement* stm = iter->first;
 			vector<Fact*>& facts1 = iter->second;
-			const FactVec& facts2 = map_facts_in[stm]; 
+			const FactVec& facts2 = map_facts_in[stm];
 			combine_facts(facts1, facts2);
-		}    
+		}
 		for(iter = map_facts_out_final.begin(); iter != map_facts_out_final.end(); ++iter) {
 			const Statement* stm = iter->first;
 			vector<Fact*>& facts1 = iter->second;
 			const FactVec& facts2 = map_facts_out[stm];
 			combine_facts(facts1, facts2);
-		}     
+		}
 	}
 	//JYTODO: beef up the sanity check
 	//sanity_check_map();
 }
 
-void 
+void
 FactMgr::set_fact_in(const Statement* s, const FactVec& facts)
 {
 	map_facts_in[s] = facts;
 }
 
-/* 
+/*
  * special treatment for "contine" and "break", their output env should has no facts
  * of local variables in the loop. For return, there should be no fact for local varibles
  * of the function
  */
-void 
+void
 FactMgr::set_fact_out(const Statement* s, const FactVec& facts)
 {
 	if (s->eType == eContinue || s->eType == eBreak) {
@@ -267,15 +272,15 @@ FactMgr::set_fact_out(const Statement* s, const FactVec& facts)
 	}
 	else if (s->eType == eGoto) {
 		const StatementGoto* sg = dynamic_cast<const StatementGoto*>(s);
-		FactVec facts_copy; 
-		FactMgr::update_facts_for_dest(facts, facts_copy, sg->dest); 
+		FactVec facts_copy;
+		FactMgr::update_facts_for_dest(facts, facts_copy, sg->dest);
 		map_facts_out[s] = facts_copy;
 	}
 	else if (s->eType == eReturn || s->parent==NULL) {
 		FactVec facts_copy = facts;
 		remove_function_local_facts(facts_copy, s);
 		map_facts_out[s] = facts_copy;
-	} 
+	}
 	else {
 		map_facts_out[s] = facts;
 	}
@@ -286,7 +291,7 @@ FactMgr::set_fact_out(const Statement* s, const FactVec& facts)
  * if it becomes irrelevant at the end of this statement.
  * such as local facts after return
  */
-void 
+void
 FactMgr::add_fact_out(const Statement* stm, const Fact* fact)
 {
 	const Variable* var = fact->get_var();
@@ -317,7 +322,7 @@ FactMgr::add_fact_out(const Statement* stm, const Fact* fact)
  * exception:	facts can be indirectly accessed through pointers
  *				for example: { int i; func(&i)}. The facts of i will not be removed
  */
-void 
+void
 FactMgr::caller_to_callee_handover(const FunctionInvocationUser* fiu, std::vector<const Fact*>& inputs)
 {
 	// add parameter facts
@@ -337,18 +342,18 @@ FactMgr::caller_to_callee_handover(const FunctionInvocationUser* fiu, std::vecto
 		}
 	}
 	// find all the facts for variables that might be pointed to by variables we already found.
-	// this include variables on stack (most likely locals of callers) but invisible to 
+	// this include variables on stack (most likely locals of callers) but invisible to
 	// this function
 	do {
-		cnt = keep_facts.size(); 
+		cnt = keep_facts.size();
 		for (i=0; i<len; i++) {
-			const Fact* f = inputs[i]; 
-			// const Variable* v = f->get_var(); 
+			const Fact* f = inputs[i];
+			// const Variable* v = f->get_var();
 			for (j=0; j<keep_facts.size(); j++) {
 				if (keep_facts[j]->eCat == ePointTo) {
 					const FactPointTo* fp = dynamic_cast<const FactPointTo*>(keep_facts[j]);
 					if (fp->point_to(f->get_var())) {
-						keep_facts.push_back(f); 
+						keep_facts.push_back(f);
 						inputs.erase(inputs.begin() + i);
 						i--;
 						len--;
@@ -357,7 +362,7 @@ FactMgr::caller_to_callee_handover(const FunctionInvocationUser* fiu, std::vecto
 				}
 			}
 		}
-	} while (keep_facts.size() > cnt); 
+	} while (keep_facts.size() > cnt);
 	inputs = keep_facts;
 }
 
@@ -367,32 +372,32 @@ FactMgr::caller_to_callee_handover(const FunctionInvocationUser* fiu, std::vecto
 void FactMgr::remove_rv_facts(FactVec& facts)
 {
 	size_t len = facts.size();
-	for (size_t i=0; i<len; i++) { 
-		const Fact* f = facts[i]; 
+	for (size_t i=0; i<len; i++) {
+		const Fact* f = facts[i];
 		if (f->get_var()->is_rv() && !func->rv->match(f->get_var())) {
 			facts.erase(facts.begin() + i);
 			len--;
-			i--; 
+			i--;
 		}
 	}
 }
 
-bool 
+bool
 FactMgr::update_fact_for_assign(const Lhs* lhs, const Expression* rhs, FactVec& inputs)
 {
 	bool changed = false;
     for (size_t i=0; i<FactMgr::meta_facts.size(); i++) {
         vector<const Fact*> facts = FactMgr::meta_facts[i]->abstract_fact_for_assign(inputs, lhs, rhs);
-		if (facts.size() == 1 && !facts[0]->get_var()->isArray) { 
+		if (facts.size() == 1 && !facts[0]->get_var()->isArray) {
 			// for must-point-to fact concerning no-array variable, just renew the old fact
-			renew_fact(inputs, facts[0]); 
+			renew_fact(inputs, facts[0]);
 		}
-		else {                   
+		else {
 			// for may-point-to facts (which means pointer on LHS is uncertain), merge them with old facts
 			for (size_t j=0; j<facts.size(); j++) {
 				const Fact* f = facts[j];
 				merge_fact(inputs, f);
-			} 
+			}
 		}
 		if (facts.size() > 0) {
 			changed = true;
@@ -401,17 +406,17 @@ FactMgr::update_fact_for_assign(const Lhs* lhs, const Expression* rhs, FactVec&
 	return changed;
 }
 
-bool 
+bool
 FactMgr::update_fact_for_assign(const StatementAssign* sa, FactVec& inputs)
 {
-	if (FactMgr::update_fact_for_assign(sa->get_lhs(), sa->get_rhs(), inputs)) { 
+	if (FactMgr::update_fact_for_assign(sa->get_lhs(), sa->get_rhs(), inputs)) {
 		sa->func->fact_changed = true;
 		return true;
     }
 	return false;
 }
 
-void 
+void
 FactMgr::update_fact_for_return(const StatementReturn* sr, FactVec& inputs)
 {
 	size_t i, j;
@@ -438,11 +443,11 @@ FactMgr::update_facts_for_dest(const FactVec& facts_in, FactVec& facts_out, cons
 	assert(func);
 	// find all the variales that are out-of-scope after jump to destination
 	// oos variables are those not on stack and not global
-	for (i=0; i<facts_in.size(); i++) { 
+	for (i=0; i<facts_in.size(); i++) {
 		const Fact* f = facts_in[i];
 		const Variable* var = f->get_var();
 		// return variable, target site don't need them
-		if (!var || var->is_rv()) continue; 
+		if (!var || var->is_rv()) continue;
 		if (func->is_var_oos(var, dest)) {
 			if (find_variable_in_set(oos_vars, var) == -1) {
 				oos_vars.push_back(var);
@@ -467,15 +472,15 @@ FactMgr::update_facts_for_dest(const FactVec& facts_in, FactVec& facts_out, cons
 ///////////////////////////////////////////////////////////////////////////////
 
 /*
- * 
+ *
  */
 FactMgr::FactMgr(const Function* f)
 : func(f)
-{ 
+{
 }
 
 /*
- * 
+ *
  */
 FactMgr::~FactMgr(void)
 {
@@ -499,39 +504,39 @@ FactMgr::add_interested_facts(int interests)
 	if (interests & eUnionWrite) {
 		FactUnion *fu = FactUnion::make_fact(0, 0);
 		meta_facts.push_back(fu);
-    } 
-} 
+    }
+}
 
-void 
+void
 FactMgr::restore_facts(vector<const Fact*>& old_facts)
 {
 	makeup_new_var_facts(old_facts, global_facts);
 	global_facts = old_facts;
 }
 
-void 
+void
 FactMgr::makeup_new_var_facts(vector<const Fact*>& old_facts, const vector<const Fact*>& new_facts)
 {
-    size_t i; 
-    for (i=0; i<new_facts.size(); i++) { 
+    size_t i;
+    for (i=0; i<new_facts.size(); i++) {
 		const Fact* f = new_facts[i];
 		const Variable* v = f->get_var();
-		if (v->is_global() || v->is_local()) { 
+		if (v->is_global() || v->is_local()) {
 			// if there are variable facts not present in old facts,
 			// mean they are variables created after old_facts,
 			// manually add them
-			if (find_related_fact(old_facts, f) == 0) { 
+			if (find_related_fact(old_facts, f) == 0) {
 				FactMgr::add_new_var_fact(v, old_facts);
 			}
 		}
 	}
 }
 
-void 
+void
 FactMgr::clear_map_visited(void)
 {
 	map<const Statement*, bool>::iterator iter;
-	for(iter = map_visited.begin(); iter != map_visited.end(); ++iter) {  
+	for(iter = map_visited.begin(); iter != map_visited.end(); ++iter) {
 		iter->second = false;
 	}
 }
@@ -541,7 +546,7 @@ FactMgr::backup_stm_fact_maps(const Statement* stm, map<const Statement*, FactVe
 {
 	vector<const Block*> blks;
 	stm->get_blocks(blks);
-	for (size_t i=0; i<blks.size(); i++) { 
+	for (size_t i=0; i<blks.size(); i++) {
 		const Block* b = blks[i];
 		facts_in[b] = map_facts_in[b];
 		facts_out[b] = map_facts_out[b];
@@ -558,7 +563,7 @@ FactMgr::restore_stm_fact_maps(const Statement* stm, map<const Statement*, FactV
 {
 	vector<const Block*> blks;
 	stm->get_blocks(blks);
-	for (size_t i=0; i<blks.size(); i++) { 
+	for (size_t i=0; i<blks.size(); i++) {
 		const Block* b = blks[i];
 		map_facts_in[b] = facts_in[b];
 		map_facts_out[b] = facts_out[b];
@@ -579,7 +584,7 @@ FactMgr::reset_stm_fact_maps(const Statement* stm)
 	FactVec empty;
 	vector<const Block*> blks;
 	stm->get_blocks(blks);
-	for (size_t i=0; i<blks.size(); i++) { 
+	for (size_t i=0; i<blks.size(); i++) {
 		const Block* b = blks[i];
 		map_facts_in[b] = empty;
 		map_facts_out[b] = empty;
@@ -593,18 +598,18 @@ FactMgr::reset_stm_fact_maps(const Statement* stm)
 
 bool
 FactMgr::merge_jump_facts(FactVec& facts, const FactVec& jump_facts)
-{ 
+{
     size_t i;
     bool changed = false;
     for (i=0; i<facts.size(); i++) {
-		const Fact* f = facts[i]; 
+		const Fact* f = facts[i];
 		if (!f->get_var()->is_rv()) {
 			const Fact* jump_f = find_related_fact(jump_facts, f);
 			// this should not happen: jump over initializers
 			if (jump_f == 0) {
 				if (f->eCat == ePointTo) {
 					jump_f = FactPointTo::make_fact(f->get_var(), FactPointTo::garbage_ptr);
-				} 
+				}
 				else if (f->eCat == eUnionWrite) {
 					jump_f = FactUnion::make_fact(f->get_var(), FactUnion::BOTTOM);
 				}
@@ -613,14 +618,14 @@ FactMgr::merge_jump_facts(FactVec& facts, const FactVec& jump_facts)
 				changed = true;
 			}
 		}
-    } 
+    }
     return changed;
 }
 
 /*
  * create a control flow graph edge introduced by break/continue/goto
  */
-void 
+void
 FactMgr::create_cfg_edge(const Statement* src, const Statement* dest, bool post_dest, bool is_back_link)
 {
 	CFGEdge* edge = new CFGEdge(src, dest, post_dest, is_back_link);
@@ -630,20 +635,20 @@ FactMgr::create_cfg_edge(const Statement* src, const Statement* dest, bool post_
 void
 FactMgr::remove_loop_local_facts(const Statement* s, FactVec& facts)
 {
-	// filter out out-of-scope facts 
-	const Block* b = (s->eType==eBlock) ? (const Block*)s : s->parent; 
+	// filter out out-of-scope facts
+	const Block* b = (s->eType==eBlock) ? (const Block*)s : s->parent;
 	vector<Variable*> local_vars = b->local_vars;
 	while (b && !b->looping) {
 		b = b->parent;
 		local_vars.insert(local_vars.end(), b->local_vars.begin(), b->local_vars.end());
-	} 
+	}
 	FactMgr::update_facts_for_oos_vars(local_vars, facts);
 }
 
 void
 FactMgr::output_assertions(std::ostream &out, const Statement* stm, int indent, bool post_condition)
 {
-	vector<Fact*> facts;  
+	vector<Fact*> facts;
 	if (!post_condition) {
 		facts = map_facts_in_final[stm];
 	} else {
@@ -653,13 +658,13 @@ FactMgr::output_assertions(std::ostream &out, const Statement* stm, int indent,
 
 	if (stm->eType == eFor || stm->eType == eIfElse) {
 		output_tab(out, indent);
-		std::ostringstream ss; 
+		std::ostringstream ss;
 		ss << "facts after " << (stm->eType == eFor ? "for loop" : "branching");
 		output_comment_line(out, ss.str());
 	}
 	if (stm->eType == eAssign || stm->eType == eInvoke || stm->eType == eReturn) {
 		output_tab(out, indent);
-		std::ostringstream ss; 
+		std::ostringstream ss;
 		ss << "statement id: " << stm->stm_id;
 		output_comment_line(out, ss.str());
 	}
@@ -672,17 +677,17 @@ FactMgr::output_assertions(std::ostream &out, const Statement* stm, int indent,
 		if (v->is_global() && !eff.is_read(v) && !eff.is_written(v)) {
 			continue;
 		}
-		output_tab(out, indent); 
+		output_tab(out, indent);
 		f->OutputAssertion(out, stm);
 	}
 }
 
-void 
+void
 FactMgr::find_updated_facts(const Statement* stm, vector<const Fact*>& facts)
 {
-	const FactVec& facts_in = map_facts_in[stm]; 
-	const FactVec& facts_out = map_facts_out[stm]; 
-	  
+	const FactVec& facts_in = map_facts_in[stm];
+	const FactVec& facts_out = map_facts_out[stm];
+
 	for (size_t i=0; i<facts_out.size(); i++) {
 		const Fact* f = facts_out[i];
 		const Fact* prev_f = find_related_fact(facts_in, f);
@@ -693,19 +698,19 @@ FactMgr::find_updated_facts(const Statement* stm, vector<const Fact*>& facts)
 	}
 }
 
-void 
+void
 FactMgr::find_updated_final_facts(const Statement* stm, vector<Fact*>& facts)
 {
-	vector<Fact*>& facts_in = map_facts_in_final[stm]; 
-	vector<Fact*>& facts_out = map_facts_out_final[stm]; 
-	  
+	vector<Fact*>& facts_in = map_facts_in_final[stm];
+	vector<Fact*>& facts_out = map_facts_out_final[stm];
+
 	for (size_t i=0; i<facts_out.size(); i++) {
 		Fact* f = facts_out[i];
 		// sometimes there is no pre-facts for return variables, so we don't
 		// check the difference
 		if (func->rv->match(f->get_var())) {
 			facts.push_back(f);
-		} 
+		}
 		else {
 			const Fact* prev_f = find_related_fact(facts_in, f);
 			assert(prev_f);
@@ -719,12 +724,12 @@ FactMgr::find_updated_final_facts(const Statement* stm, vector<Fact*>& facts)
 void
 FactMgr::find_dangling_global_ptrs(Function* f)
 {
-	for (size_t i=0; i<global_facts.size(); i++) { 
+	for (size_t i=0; i<global_facts.size(); i++) {
 		if (global_facts[i]->eCat == ePointTo) {
 			FactPointTo* fp = (FactPointTo*)(global_facts[i]);
-			const Variable* v = fp->get_var(); 
+			const Variable* v = fp->get_var();
 			// const pointers should never be dangling
-			if (v->is_const() || !v->is_global()) continue;   
+			if (v->is_const() || !v->is_global()) continue;
 			if (fp->is_dead()) {
 				f->dead_globals.push_back(v);
 			}
@@ -735,22 +740,22 @@ FactMgr::find_dangling_global_ptrs(Function* f)
 void
 FactMgr::sanity_check_map() const
 {
-	map<const Statement*, vector<const Fact*> >::const_iterator iter; 
+	map<const Statement*, vector<const Fact*> >::const_iterator iter;
 	for(iter = map_facts_in.begin(); iter != map_facts_in.end(); ++iter) {
 		const Statement* stm = iter->first;
 		const vector<const Fact*>& facts = iter->second;
 		for (size_t i=0; i<facts.size(); i++) {
 			const Variable* v = facts[i]->get_var();
 			if (!v->is_visible(stm->parent)) {
-				// exception: the input facts to a function body could include the parameter facts 
+				// exception: the input facts to a function body could include the parameter facts
 				if (stm->parent == 0 && find_variable_in_set(func->param, v) != -1) {
 					continue;
 				}
 				//assert(0);
 			}
 		}
-	} 
-		
+	}
+
 	for(iter = map_facts_out.begin(); iter != map_facts_out.end(); ++iter) {
 		const Statement* stm = iter->first;
 		const vector<const Fact*>& facts = iter->second;
