@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (c) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 The University of Utah
+// Copyright (c) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017 The University of Utah
 // All rights reserved.
 //
 // This file is part of `csmith', a random generator of C programs.
@@ -72,9 +72,14 @@ FUTURE:
 	- Choose random functions to perform allocations
 	- Choose random children/ancestors to perform deallocations
 	- Work from leaves to root
-	- If node uses pointer or array, it is potential heap store allocated. 
+	- If node uses pointer or array, it is potential heap store allocated.
 */
-#ifdef WIN32 
+
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#ifdef WIN32
 #pragma warning(disable : 4786)   /* Disable annoying warning messages */
 #endif
 
@@ -88,6 +93,7 @@ FUTURE:
 #include "CGOptions.h"
 #include "AbsProgramGenerator.h"
 
+#include "git_version.h"
 #include "platform.h"
 #include "random.h"
 
@@ -108,9 +114,7 @@ static void
 print_version(void)
 {
 	cout << PACKAGE_STRING << endl;
-#ifdef GIT_VERSION
-	cout << "Git version: " << GIT_VERSION << endl;
-#endif
+	cout << "Git version: " << git_version << endl;
 	// XXX print copyright, contact info, etc.?
 }
 
@@ -119,7 +123,7 @@ print_version(void)
 bool parse_string_arg(const char *arg, string &s)
 {
 	s.assign(arg);
-	return ((!s.empty()) && 
+	return ((!s.empty()) &&
 		(s.compare(0, 2, "--")));
 }
 
@@ -128,7 +132,7 @@ parse_int_arg(char *arg, unsigned long *ret)
 {
 	int res;
 	res = sscanf (arg, "%lu", ret);
-	
+
 	if (res == 0) {
 		cout << "expected integer at arg position " << endl;
 		return false;
@@ -138,21 +142,21 @@ parse_int_arg(char *arg, unsigned long *ret)
 
 static void print_help()
 {
-	cout << "Command line options: " << endl << endl; 
+	cout << "Command line options: " << endl << endl;
 	// most common options
-	cout << "  --help or -h: print this information." << endl << endl; 
-	cout << "  -hh: describe extra options probably useful only for Csmith developers." << endl << endl; 
-	cout << "  --version or -v: print the version of Csmith." << endl << endl; 
+	cout << "  --help or -h: print this information." << endl << endl;
+	cout << "  -hh: describe extra options probably useful only for Csmith developers." << endl << endl;
+	cout << "  --version or -v: print the version of Csmith." << endl << endl;
 	cout << "  --seed <seed> or -s <seed>: use <seed> instead of a random seed generated by Csmith." << endl << endl;
 	cout << "  --output <filename> or -o <filename>: specify the output file name." << endl << endl;
 
 	// enabling/disabling options
-	cout << "  --argc | --no-argc: genereate main function with/without argv and argc being passed (enabled by default)." << endl << endl; 
+	cout << "  --argc | --no-argc: genereate main function with/without argv and argc being passed (enabled by default)." << endl << endl;
 	cout << "  --arrays | --no-arrays: enable | disable arrays (enabled by default)." << endl << endl;
-	cout << "  --bitfields | --no-bitfields: enable | disable full-bitfields structs (disabled by default)." << endl << endl;
+	cout << "  --bitfields | --no-bitfields: enable | disable full-bitfields structs (enabled by default)." << endl << endl;
 	cout << "  --checksum | --no-checksum: enable | disable checksum calculation (enabled by default)." << endl << endl;
 	cout << "  --comma-operators | --no-comma-operators: enable | disable comma operators (enabled by default)." << endl << endl;
-	cout << "  --compound-assignment | --no-compound-assignment: enable | disable compound assignments (enabled by default)." << endl << endl; 
+	cout << "  --compound-assignment | --no-compound-assignment: enable | disable compound assignments (enabled by default)." << endl << endl;
 	cout << "  --concise: generated programs with minimal comments (disabled by default)." << endl << endl;
 	cout << "  --consts | --no-consts: enable | disable const qualifier (enabled by default)." << endl << endl;
 	cout << "  --divs | --no-divs: enable | disable divisions (enabled by default)." << endl << endl;
@@ -167,16 +171,16 @@ static void print_help()
 	cout << "  --int8 | --no-int8: enable | disable int8_t (enabled by default)." << endl << endl;
 	cout << "  --uint8 | --no-uint8: enable | disable uint8_t (enabled by default)." << endl << endl;
 	cout << "  --float | --no-float: enable | disable float (disabled by default)." << endl << endl;
-	cout << "  --main | --nomain: enable | disable to generate main function (enabled by default)." << endl <<  endl;  
+	cout << "  --main | --nomain: enable | disable to generate main function (enabled by default)." << endl <<  endl;
 	cout << "  --math64 | --no-math64: enable | disable 64-bit math ops (enabled by default)." << endl << endl;
 	cout << "  --inline-function | --no-inline-function: enable | disable inline attributes on generated functions." << endl << endl;
 	cout << "  --inline-function-prob <num>: set the probability of each function being marked as inline (default is 50)." << endl << endl;
 
-	// numbered controls	
+	// numbered controls
 	cout << "  --max-array-dim <num>: limit array dimensions to <num>. (default 3)" << endl << endl;
 	cout << "  --max-array-len-per-dim <num>: limit array length per dimension to <num> (default 10)." << endl << endl;
 	cout << "  --max-block-depth <num>: limit depth of nested blocks to <num> (default 5)." << endl << endl;
-	cout << "  --max-block-size <size>: limit the number of non-return statements in a block to <size> (default 4)." << endl << endl; 
+	cout << "  --max-block-size <size>: limit the number of non-return statements in a block to <size> (default 4)." << endl << endl;
 	cout << "  --max-expr-complexity <num>: limit expression complexities to <num> (default 10)." << endl << endl;
 	cout << "  --max-funcs <num>: limit the number of functions (besides main) to <num>  (default 10)." << endl << endl;
 	cout << "  --max-pointer-depth <depth>: limit the indirect depth of pointers to <depth> (default 2)." << endl << endl;
@@ -185,15 +189,16 @@ static void print_help()
 
 	cout << "  --muls | --no-muls: enable | disable multiplications (enabled by default)." << endl << endl;
 	cout << "  --safe-math | --no-safe-math: Emit safe math wrapper functions (enabled by default)." << endl << endl;
-	cout << "  --packed-struct | --no-packed-struct: enable | disable packed structs by adding #pragma pack(1) before struct definition (disabled by default)." << endl << endl; 
-	cout << "  --paranoid | --no-paranoid: enable | disable pointer-related assertions (disabled by default)." << endl << endl; 
+	cout << "  --packed-struct | --no-packed-struct: enable | disable packed structs by adding #pragma pack(1) before struct definition (enabled by default)." << endl << endl;
+	cout << "  --paranoid | --no-paranoid: enable | disable pointer-related assertions (disabled by default)." << endl << endl;
 	cout << "  --pointers | --no-pointers: enable | disable pointers (enabled by default)." << endl << endl;
-	cout << "  --quiet: generate programs with less comments (disabled by default)." << endl << endl; 
+	cout << "  --quiet: generate programs with less comments (disabled by default)." << endl << endl;
 	cout << "  --structs | --no-structs: enable | disable to generate structs (enable by default)." << endl << endl;
 	cout << "  --unions | --no-unions: enable | disable to generate unions (enable by default)." << endl << endl;
 	cout << "  --volatiles | --no-volatiles: enable | disable volatiles (enabled by default)." << endl << endl;
 	cout << "  --volatile-pointers | --no-volatile-pointers: enable | disable volatile pointers (enabled by default)." << endl << endl;
 	cout << "  --const-pointers | --no-const-pointers: enable | disable const pointers (enabled by default)." << endl << endl;
+	cout << "  --global-variables | --no-global-variables: enable | disable global variables (enabled by default)." << endl << endl;
 
 	cout << "  --builtins | --no-builtins: enable | disable to generate builtin functions (disabled by default)." << endl << endl;
 	cout << "  --enable-builtin-kinds k1,k2 | --disable-builtin-kinds k1,k2: enable | disable certain kinds of builtin functions." << endl << endl;
@@ -201,22 +206,23 @@ static void print_help()
 
         // language options
 	cout << "  --lang-cpp : generate C++ code (C by default)." << endl << endl;
+	cout << "  --cpp11 : generate C++11 code (C++03 by default). Works if lang-cpp is enabled." << endl << endl;
 
 }
 
 static void print_advanced_help()
 {
 	cout << "'Advanced' command line options that are probably only useful for Csmith's" << endl;
-	cout << "original developers:" << endl << endl; 
+	cout << "original developers:" << endl << endl;
 	// file split options
 	cout << "  --max-split-files <num>: evenly split a generated program into <num> different files(default 0)." << endl << endl;
-	cout << "  --split-files-dir <dir>: generate split-files into <dir> (default ./output)." << endl << endl; 
+	cout << "  --split-files-dir <dir>: generate split-files into <dir> (default ./output)." << endl << endl;
 
 	// dfs-exhaustive mode options
 	cout << "  --dfs-exhaustive: enable depth first exhaustive random generation (disabled by default)." << endl << endl;
 	cout << "  --expand-struct: enable the expansion of struct in the exhaustive mode. ";
-	cout << "Only works in the exhaustive mode and cannot used with --no-structs." << endl << endl; 
-	
+	cout << "Only works in the exhaustive mode and cannot used with --no-structs." << endl << endl;
+
 	cout << "  --compact-output: print generated programs in compact way. ";
 	cout << "Only works in the exhaustive mode." << endl << endl;
 
@@ -236,34 +242,35 @@ static void print_advanced_help()
 	cout << "Only works in the exhaustive mode." << endl << endl;
 
 	// target platforms
-	cout << "  --msp: enable certain msp related features " << endl << endl; 
+	cout << "  --msp: enable certain msp related features " << endl << endl;
 	cout << "  --ccomp: generate compcert-compatible code" << endl << endl;
 
 	// symblic excutions
-	cout << "  --splat: enable splat extension" << endl << endl; 
-	cout << "  --klee: enable klee extension" << endl << endl; 
-	cout << "  --crest: enable crest extension" << endl << endl;  
+	cout << "  --splat: enable splat extension" << endl << endl;
+	cout << "  --klee: enable klee extension" << endl << endl;
+	cout << "  --crest: enable crest extension" << endl << endl;
 
 	// coverage test options
-	cout << "  --coverage-test: enable coverage-test extension" << endl << endl; 
+	cout << "  --coverage-test: enable coverage-test extension" << endl << endl;
 	cout << "  --coverage-test-size <num>: specify size (default 500) of the array generated to test coverage. ";
-	cout << "Can only be used with --coverage-test." << endl << endl; 
+	cout << "Can only be used with --coverage-test." << endl << endl;
 
 	cout << "  --func1_max_params <num>: specify the number of symbolic variables passed to func_1 (default 3). ";
 	cout << "Only used when --splat | --crest | --klee | --coverage-test is enabled." << endl << endl;
-	 
+
 	// struct/union related options
-	cout << "  --fixed-struct-fields: fix the size of struct fields to max-struct-fields (default 10)." << endl << endl; 
-	cout << "  --return-structs | --no-return-structs: enable | disable return structs from a function (enabled by default)." << endl << endl; 
-	cout << "  --arg-structs | --no-arg-structs: enable | disable structs being used as args (enabled by default)." << endl << endl; 
-	cout << "  --return-unions | --no-return-unions: enable | disable return unions from a function (enabled by default)." << endl << endl; 
-	cout << "  --arg-unions | --no-arg-unions: enable | disable unions being used as args (enabled by default)." << endl << endl; 
-	cout << "  --take-union-field-addr | --take-no-union-field-addr: allow | disallow addresses of union fields to be taken (allowed by default)." << endl << endl; 
+	cout << "  --fixed-struct-fields: fix the size of struct fields to max-struct-fields (default 10)." << endl << endl;
+	cout << "  --return-structs | --no-return-structs: enable | disable return structs from a function (enabled by default)." << endl << endl;
+	cout << "  --arg-structs | --no-arg-structs: enable | disable structs being used as args (enabled by default)." << endl << endl;
+	cout << "  --return-unions | --no-return-unions: enable | disable return unions from a function (enabled by default)." << endl << endl;
+	cout << "  --arg-unions | --no-arg-unions: enable | disable unions being used as args (enabled by default)." << endl << endl;
+	cout << "  --take-union-field-addr | --take-no-union-field-addr: allow | disallow addresses of union fields to be taken (allowed by default)." << endl << endl;
 	cout << "  --vol-struct-union-fields | --no-vol-struct-union-fields: enable | disable volatile struct/union fields (enabled by default)" << endl << endl;
+	cout << "  --const-struct-union-fields | --no-const-struct-union-fields: enable | disable const struct/union fields (enabled by default)" << endl << endl;
 
 	// delta related options
 	cout << "  --delta-monitor [simple]: specify the type of delta monitor. Only [simple] type is supported now." << endl << endl;
-	cout << "  --delta-input [file]: specify the file for delta input." << endl << endl; 
+	cout << "  --delta-input [file]: specify the file for delta input." << endl << endl;
 	cout << "  --delta-output [file]: specify the file for delta output (default to <delta-input>)." << endl << endl;
 	cout << "  --go-delta [simple]: run delta reduction on <delta-input>." << endl << endl;
 	cout << "  --no-delta-reduction: output the same program as <delta-input>. ";
@@ -286,9 +293,9 @@ static void print_advanced_help()
 
 	// other options
 	cout << "  --math-notmp: make csmith generate code for safe_math_macros_notmp." << endl << endl;
-	 
+
 	cout << "  --strict-const-arrays: restrict array elements to constants." << endl << endl;
-	
+
 	cout << "  --partial-expand <assignment[,for[,block[,if-else[,invoke[,return]]]]]: ";
 	cout <<"partial-expand controls which what kind of statements should be generated first. ";
 	cout <<"For example, it could make Csmith start to generate if-else without go over assignment or for." << endl << endl;
@@ -300,35 +307,36 @@ static void print_advanced_help()
 	cout << "  --monitor-funcs <name1,name2...>: dump the checksums after each statement in the monitored functions." << endl << endl;
 
 	cout << "  --step-hash-by-stmt: dump the checksum after each statement. It is applied to all functions unless --monitor-funcs is specified." << endl << endl;
-	
+
 	cout << "  --stop-by-stmt <num>: try to stop generating statements after the statement with id <num>." << endl << endl;
-	
+
 	cout << "  --const-as-condition: enable const to be conditions of if-statements. " << endl << endl;
 
 	cout << "  --match-exact-qualifiers: match exact const/volatile qualifiers for LHS and RHS of assignments." << endl << endl;
-	
+
 	cout << "  --reduce <file>: reduce random program under the direction of the configuration file." << endl << endl;
 
 	cout << "  --return-dead-pointer | --no-return-dead-pointer: allow | disallow functions from returning dangling pointers (disallowed by default)." << endl << endl;
 
 	cout <<	"  --identify-wrappers: assign ids to used safe math wrappers." << endl << endl;
-		 
-	cout << "  --safe-math-wrappers <id1,id2...>: specifiy ids of wrapper functions that are necessary to avoid undefined behaviors, use 0 to specify none." << endl << endl;
-				
-	cout << "  --mark-mutable-const: mark constants that can be mutated with parentheses (disabled by default)." << endl << endl; 
 
-	cout << "  --force-non-uniform-arrays | --no-force-non-uniform-arrays: force integer arrays to be initialized with multiple values (enabled by default)." << endl << endl; 
- 
+	cout << "  --safe-math-wrappers <id1,id2...>: specifiy ids of wrapper functions that are necessary to avoid undefined behaviors, use 0 to specify none." << endl << endl;
+
+	cout << "  --mark-mutable-const: mark constants that can be mutated with parentheses (disabled by default)." << endl << endl;
+
+	cout << "  --force-non-uniform-arrays | --no-force-non-uniform-arrays: force integer arrays to be initialized with multiple values (enabled by default)." << endl << endl;
+
 	cout << "  --null-ptr-deref-prob <N>: allow null pointers to be dereferenced with probability N% (0 by default)." << endl << endl;
 
-	cout << "  --dangling-ptr-deref-prob <N>: allow dangling pointers to be dereferenced with probability N% (0 by default)." << endl << endl; 
+	cout << "  --dangling-ptr-deref-prob <N>: allow dangling pointers to be dereferenced with probability N% (0 by default)." << endl << endl;
 
 	cout << "  --union-read-type-sensitive | --no-union-read-type-sensitive: allow | disallow reading an union field when there is no risk of "
-		 << "reading padding bits (enabled by default)." << endl << endl; 
+		 << "reading padding bits (enabled by default)." << endl << endl;
 
 	cout << "  --max-struct-nested-level: controls the max depth of nested structs (default is 3)." << endl << endl;
 	cout << "  --no-hash-value-printf: do not emit printf on the index of an array" << endl << endl;
 	cout << "  --no-signed-char-index: do not allow a var of type char to be used as array index" << endl << endl;
+	cout << "  --strict-float: do not allow assignments between floats and integers" << endl << endl;
 }
 
 void arg_check(int argc, int i)
@@ -346,7 +354,7 @@ main(int argc, char **argv)
 	g_Seed = platform_gen_seed();
 
 	CGOptions::set_default_settings();
-			 
+
 	for (int i=1; i<argc; i++) {
 
 		if (strcmp (argv[i], "--help") == 0 ||
@@ -365,7 +373,7 @@ main(int argc, char **argv)
 			print_version();
 			return 0;
 		}
-		
+
 		if (strcmp (argv[i], "--seed") == 0 ||
 			strcmp (argv[i], "-s") == 0) {
 			i++;
@@ -787,6 +795,11 @@ main(int argc, char **argv)
 			continue;
 		}
 
+		if (strcmp (argv[i], "--strict-float") == 0) {
+			CGOptions::strict_float(true);
+			continue;
+		}
+
 		if (strcmp (argv[i], "--pointers") == 0) {
 			CGOptions::pointers(true);
 			continue;
@@ -912,6 +925,16 @@ main(int argc, char **argv)
 			continue;
 		}
 
+		if (strcmp (argv[i], "--global-variabless") == 0) {
+			CGOptions::global_variables(true);
+			continue;
+		}
+
+		if (strcmp (argv[i], "--no-global-variables") == 0) {
+			CGOptions::global_variables(false);
+			continue;
+		}
+
 		if (strcmp (argv[i], "--enable-access-once") == 0) {
 			CGOptions::access_once(true);
 			continue;
@@ -985,7 +1008,7 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--no-checksum") == 0) {
 			CGOptions::compute_hash(false);
 			continue;
-		} 
+		}
 
 		if (strcmp (argv[i], "--builtins") == 0) {
 			CGOptions::builtins(true);
@@ -1001,7 +1024,7 @@ main(int argc, char **argv)
 			CGOptions::random_random(true);
 			continue;
 		}
-		
+
 		if (strcmp (argv[i], "--check-global") == 0) {
 			CGOptions::blind_check_global(true);
 			continue;
@@ -1021,7 +1044,7 @@ main(int argc, char **argv)
 			CGOptions::stop_by_stmt(num);
 			continue;
 		}
-		
+
 		if (strcmp (argv[i], "--monitor-funcs") == 0) {
 			string vname;
 			i++;
@@ -1033,11 +1056,6 @@ main(int argc, char **argv)
 			CGOptions::monitored_funcs(vname);
 			continue;
 		}
-		
-		if (strcmp (argv[i], "--deputy") == 0) {
-			CGOptions::deputy(true);
-			continue;
-		}
 
 		if (strcmp (argv[i], "--delta-input") == 0) {
 			string filename;
@@ -1090,7 +1108,7 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--const-as-condition") == 0) {
 			CGOptions::const_as_condition(true);
 			continue;
-		} 
+		}
 
 		if (strcmp (argv[i], "--match-exact-qualifiers") == 0) {
 			CGOptions::match_exact_qualifiers(true);
@@ -1263,7 +1281,7 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--no-pre-incr-operator") == 0) {
 			CGOptions::pre_incr_operator(false);
 			continue;
-		} 
+		}
 
 		if (strcmp (argv[i], "--pre-decr-operator") == 0) {
 			CGOptions::pre_decr_operator(true);
@@ -1273,7 +1291,7 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--no-pre-decr-operator") == 0) {
 			CGOptions::pre_decr_operator(false);
 			continue;
-		} 
+		}
 
 		if (strcmp (argv[i], "--post-incr-operator") == 0) {
 			CGOptions::post_incr_operator(true);
@@ -1283,7 +1301,7 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--no-post-incr-operator") == 0) {
 			CGOptions::post_incr_operator(false);
 			continue;
-		} 
+		}
 
 		if (strcmp (argv[i], "--post-decr-operator") == 0) {
 			CGOptions::post_decr_operator(true);
@@ -1293,7 +1311,7 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--no-post-decr-operator") == 0) {
 			CGOptions::post_decr_operator(false);
 			continue;
-		} 
+		}
 
 		if (strcmp (argv[i], "--unary-plus-operator") == 0) {
 			CGOptions::unary_plus_operator(true);
@@ -1303,7 +1321,7 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--no-unary-plus-operator") == 0) {
 			CGOptions::unary_plus_operator(false);
 			continue;
-		} 
+		}
 
 		if (strcmp (argv[i], "--embedded-assigns") == 0) {
 			CGOptions::use_embedded_assigns(true);
@@ -1323,8 +1341,8 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--no-embedded-assigns") == 0) {
 			CGOptions::use_embedded_assigns(false);
 			continue;
-		} 
-	
+		}
+
 		if (strcmp (argv[i], "--comma-operators") == 0) {
 			CGOptions::use_comma_exprs(true);
 			continue;
@@ -1333,7 +1351,7 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--no-comma-operators") == 0) {
 			CGOptions::use_comma_exprs(false);
 			continue;
-		}  
+		}
 
 		if (strcmp (argv[i], "--take-no-union-field-addr") == 0) {
 			CGOptions::take_union_field_addr(false);
@@ -1343,17 +1361,27 @@ main(int argc, char **argv)
 		if (strcmp (argv[i], "--take-union-field-addr") == 0) {
 			CGOptions::take_union_field_addr(true);
 			continue;
-		}  
+		}
 
 		if (strcmp (argv[i], "--vol-struct-union-fields") == 0) {
 			CGOptions::vol_struct_union_fields(true);
 			continue;
-		}  
+		}
 
 		if (strcmp (argv[i], "--no-vol-struct-union-fields") == 0) {
 			CGOptions::vol_struct_union_fields(false);
 			continue;
-		}  
+		}
+
+		if (strcmp(argv[i], "--const-struct-union-fields") == 0) {
+			CGOptions::const_struct_union_fields(true);
+			continue;
+		}
+
+		if (strcmp(argv[i], "--no-const-struct-union-fields") == 0) {
+			CGOptions::const_struct_union_fields(false);
+			continue;
+		}
 
 		if (strcmp (argv[i], "--no-hash-value-printf") == 0) {
 			CGOptions::hash_value_printf(false);
@@ -1370,6 +1398,11 @@ main(int argc, char **argv)
 			continue;
 		}
 
+		if (strcmp(argv[i], "--cpp11") == 0) {
+			CGOptions::cpp11(true);
+			continue;
+		}
+
 		if (strcmp (argv[i], "--reduce") == 0) {
 			string filename;
 			i++;
@@ -1386,29 +1419,42 @@ main(int argc, char **argv)
 			CGOptions::init_reducer(filename);
 			continue;
 		}
+
+    if (strcmp(argv[i], "--fast-execution") == 0) {
+      CGOptions::lang_cpp(true);
+      // jumps can easily cause infinite loops. Just disable them
+      CGOptions::jumps(false);
+      // large arrays are also reported to cause slow execution
+      CGOptions::max_array_length_per_dimension(5);
+      continue;
+    }
 		// OMIT help
 
 		// OMIT compute-hash
-		
+
 		// OMIT depth-protect
-		
+
 		// OMIT wrap-volatiles
-		
+
 		// OMIT allow-const-volatile
-		
+
 		// OMIT allow-int64
-		
+
 		// OMIT avoid-signed-overflow
 
 		// FIXME-- we should parse all options and then this should be
 		// an error
-		
+
 		cout << "invalid option " << argv[i] << " at: "
-			 << i 
+			 << i
 			 << endl;
 		exit(-1);
 	}
 
+	if (CGOptions::lang_cpp()) {
+		CGOptions::fix_options_for_cpp();
+	}
+
 	if (CGOptions::has_conflict()) {
 		cout << "error: options conflict - " << CGOptions::conflict_msg() << std::endl;
 		exit(-1);
