@@ -1,6 +1,6 @@
 // -*- mode: C++ -*-
 //
-// Copyright (c) 2007, 2008, 2010, 2011, 2013, 2014 The University of Utah
+// Copyright (c) 2007, 2008, 2010, 2011, 2013, 2014, 2015, 2016, 2017 The University of Utah
 // All rights reserved.
 //
 // This file is part of `csmith', a random generator of C programs.
@@ -36,10 +36,14 @@
 // July, 2005
 //
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "Type.h"
 #include <sstream>
-#include <assert.h>
-#include <math.h>
+#include <cassert>
+#include <cmath>
 #include "Common.h"
 #include "CGOptions.h"
 #include "random.h"
@@ -81,7 +85,7 @@ public:
 
 private:
 	mutable Type *typ_;
-	
+
 };
 
 NonVoidTypeFilter::NonVoidTypeFilter()
@@ -113,7 +117,7 @@ NonVoidTypeFilter::filter(int v) const
 		Filter *filter = SIMPLE_TYPES_PROB_FILTER;
 		return filter->filter(typ_->simple_type);
 	}
-	
+
 	return false;
 }
 
@@ -137,7 +141,7 @@ public:
 
 private:
 	mutable Type *typ_;
-	
+
 };
 
 NonVoidNonVolatileTypeFilter::NonVoidNonVolatileTypeFilter()
@@ -180,7 +184,7 @@ NonVoidNonVolatileTypeFilter::filter(int v) const
 		Filter *filter = SIMPLE_TYPES_PROB_FILTER;
 		return filter->filter(typ_->simple_type);
 	}
-	
+
 	return false;
 }
 
@@ -194,7 +198,7 @@ NonVoidNonVolatileTypeFilter::get_type()
 class ChooseRandomTypeFilter : public Filter
 {
 public:
-	ChooseRandomTypeFilter(bool for_field_var);
+	ChooseRandomTypeFilter(bool for_field_var, bool struct_has_assign_ops = false);
 
 	virtual ~ChooseRandomTypeFilter();
 
@@ -203,13 +207,15 @@ public:
 	Type *get_type();
 
 	bool for_field_var_;
-
+	bool struct_has_assign_ops_;
 private:
 	mutable Type *typ_;
 };
 
-ChooseRandomTypeFilter::ChooseRandomTypeFilter(bool for_field_var)
-  : for_field_var_(for_field_var)
+ChooseRandomTypeFilter::ChooseRandomTypeFilter(bool for_field_var, bool struct_has_assign_ops)
+  : for_field_var_(for_field_var),
+    struct_has_assign_ops_(struct_has_assign_ops),
+    typ_(NULL)
 {
 }
 
@@ -232,6 +238,12 @@ ChooseRandomTypeFilter::filter(int v) const
 		return true;
 	}
 
+	// Struct without assignment ops can not be made a field of a struct with assign ops 
+	// with current implementation of these ops
+    if (for_field_var_ && struct_has_assign_ops_ && !typ_->has_assign_ops()) {
+		assert(CGOptions::lang_cpp());
+		return true;
+	}
 	if (for_field_var_ && typ_->get_struct_depth() >= CGOptions::max_nested_struct_level()) {
 		return true;
 	}
@@ -247,6 +259,25 @@ ChooseRandomTypeFilter::get_type()
 
 ///////////////////////////////////////////////////////////////////////////////
 
+// Helper functions
+
+///////////////////////////////////////////////////////////////////////////////
+
+static bool checkImplicitNontrivialAssignOps(vector<const Type*> fields)
+{
+	if (!CGOptions::lang_cpp()) return false;
+	for (size_t i = 0; i < fields.size(); ++i) {
+		const Type* field = fields[i];
+		if (field->has_implicit_nontrivial_assign_ops()) {
+			assert((field->eType == eStruct) || (field->eType == eUnion));
+			return true;
+		}
+	}
+	return false;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
 // --------------------------------------------------------------
 /* constructor for simple types
  ********************************************************/
@@ -254,45 +285,34 @@ Type::Type(eSimpleType simple_type) :
 	eType(eSimple),
 	ptr_type(0),
 	simple_type(simple_type),
+	sid(0), // not used for simple types
 	used(false),
 	printed(false),
-        packed_(false)
+	packed_(false),
+	has_assign_ops_(false),
+	has_implicit_nontrivial_assign_ops_(false)
 {
 	// Nothing else to do.
 }
 
-// --------------------------------------------------------------
- /* copy constructor
-  *******************************************************/
-#if 0
-Type::Type(const Type &t) :
-	eType(t.eType),
-	ptr_type(t.ptr_type),
-	simple_type(t.simple_type),
-	dimensions(t.dimensions),
-	fields(t.fields),
-	used(t.used),
-	printed(t.printed)
-{
-	// Nothing else to do.
-} 
-#endif
-
 // --------------------------------------------------------------
  /* constructor for struct or union types
   *******************************************************/
-Type::Type(vector<const Type*>& struct_fields, bool isStruct, bool packed, 
-			vector<CVQualifiers> &qfers, vector<int> &fields_length) :
+Type::Type(vector<const Type*>& struct_fields, bool isStruct, bool packed,
+    vector<CVQualifiers> &qfers, vector<int> &fields_length, bool hasAssignOps, bool hasImplicitNontrivialAssignOps) :
     ptr_type(0),
+    simple_type(MAX_SIMPLE_TYPES), // not a valid simple type
     fields(struct_fields),
     used(false),
     printed(false),
     packed_(packed),
+    has_assign_ops_(hasAssignOps),
+    has_implicit_nontrivial_assign_ops_(hasImplicitNontrivialAssignOps),
     qfers_(qfers),
     bitfields_length_(fields_length)
 {
     static unsigned int sequence = 0;
-	if (isStruct) 
+	if (isStruct)
         eType = eStruct;
     else
         eType = eUnion;
@@ -303,11 +323,14 @@ Type::Type(vector<const Type*>& struct_fields, bool isStruct, bool packed,
  /* constructor for pointers
   *******************************************************/
 Type::Type(const Type* t) :
-	eType(ePointer),
+    eType(ePointer),
     ptr_type(t),
+    simple_type(MAX_SIMPLE_TYPES), // not a valid simple type
     used(false),
     printed(false),
-    packed_(false)
+    packed_(false),
+    has_assign_ops_(false),
+    has_implicit_nontrivial_assign_ops_(false)
 {
 	// Nothing else to do.
 }
@@ -331,7 +354,7 @@ Type::operator=(const Type& t)
 	simple_type = t.simple_type;
     dimensions = t.dimensions;
     fields = t.fields;
-	
+
 	return *this;
 }
 #endif
@@ -342,13 +365,15 @@ Type::get_simple_type(eSimpleType st)
 {
 	static bool inited = false;
 
+	assert(st != MAX_SIMPLE_TYPES);
+
 	if (!inited) {
 		for (int i = 0; i < MAX_SIMPLE_TYPES; ++i) {
 			Type::simple_types[i] = 0;
 		}
 		inited = true;
 	}
-	
+
 	if (Type::simple_types[st] == 0) {
 		// find if type is in the allTypes already (most likely only "eVoid" is not there)
 		for (size_t i=0; i<AllTypes.size(); i++) {
@@ -421,7 +446,7 @@ get_int_type()
 
 Type*
 Type::find_type(const Type* t)
-{ 
+{
     for (size_t i=0; i<AllTypes.size(); i++) {
         if (AllTypes[i] == t) {
             return AllTypes[i];
@@ -436,7 +461,7 @@ Type::find_type(const Type* t)
  *************************************************************/
 Type*
 Type::find_pointer_type(const Type* t, bool add)
-{ 
+{
     for (size_t i=0; i<derived_types.size(); i++) {
         if (derived_types[i]->ptr_type == t) {
             return derived_types[i];
@@ -461,10 +486,10 @@ Type::is_const_struct_union() const
 		if (field->is_const_struct_union()) {
 			return true;
 		}
-		const CVQualifiers& cf = qfers_[i]; 
+		const CVQualifiers& cf = qfers_[i];
 		if (cf.is_const()) return true;
 	}
-	return false; 
+	return false;
 }
 
 bool
@@ -478,17 +503,17 @@ Type::is_volatile_struct_union() const
 		if (field->is_volatile_struct_union()) {
 			return true;
 		}
-		const CVQualifiers& cf = qfers_[i]; 
-		if (cf.is_volatile()) 
+		const CVQualifiers& cf = qfers_[i];
+		if (cf.is_volatile())
 			return true;
 	}
-	return false; 
+	return false;
 }
 
 bool
 Type::has_int_field() const
 {
-	if (is_int()) return true; 
+	if (is_int()) return true;
 	for (size_t i=0; i<fields.size(); ++i) {
 		const Type* t = fields[i];
 		if (t->has_int_field()) return true;
@@ -496,7 +521,7 @@ Type::has_int_field() const
 	return false;
 }
 
-bool 
+bool
 Type::signed_overflow_possible() const
 {
 	return eType == eSimple && is_signed() && ((int)SizeInBytes()) >= CGOptions::int_size();
@@ -505,7 +530,7 @@ Type::signed_overflow_possible() const
 void
 Type::get_all_ok_struct_union_types(vector<Type *> &ok_types, bool no_const, bool no_volatile, bool need_int_field, bool bStruct)
 {
-	vector<Type *>::iterator i; 
+	vector<Type *>::iterator i;
 	for(i = AllTypes.begin(); i != AllTypes.end(); ++i) {
 		Type* t = (*i);
 		if (bStruct && t->eType != eStruct) continue;
@@ -519,6 +544,26 @@ Type::get_all_ok_struct_union_types(vector<Type *> &ok_types, bool no_const, boo
 	}
 }
 
+bool 
+Type::if_struct_will_have_assign_ops()
+{
+	// randomly choose if the struct will have assign operators (for C++):
+	if (!CGOptions::lang_cpp())
+		return false;
+	return rnd_flipcoin(RegularVolatileProb);
+}
+
+// To have volatile unions in C++. I am not sure if we need those
+// (If not, will be enough to return false here)
+bool
+Type::if_union_will_have_assign_ops()
+{
+	// randomly choose if the union will have assign operators (for C++):
+	if (!CGOptions::lang_cpp())
+		return false;
+	return rnd_flipcoin(RegularVolatileProb);
+}
+
 const Type*
 Type::choose_random_struct_union_type(vector<Type *> &ok_types)
 {
@@ -536,16 +581,16 @@ Type::choose_random_struct_union_type(vector<Type *> &ok_types)
 	return rv_type;
 }
 
-const Type* 
-Type::choose_random_pointer_type(void) 
+const Type*
+Type::choose_random_pointer_type(void)
 {
 	unsigned int index = rnd_upto(derived_types.size());
 	ERROR_GUARD(NULL);
 	return derived_types[index];
 }
 
-bool 
-Type::has_pointer_type(void) 
+bool
+Type::has_pointer_type(void)
 {
 	return derived_types.size() > 0;
 }
@@ -570,19 +615,19 @@ Type::choose_random_struct_from_type(const Type* type, bool no_volatile)
 	return t;
 }
 
-const Type* 
+const Type*
 Type::random_type_from_type(const Type* type, bool no_volatile, bool strict_simple_type)
 {
 	const Type* t = type;
 	DEPTH_GUARD_BY_TYPE_RETURN(dtRandomTypeFromType, NULL);
 	if (type == 0) {
-		t = no_volatile ? choose_random_nonvoid_nonvolatile() : choose_random_nonvoid(); 
+		t = no_volatile ? choose_random_nonvoid_nonvolatile() : choose_random_nonvoid();
 		ERROR_GUARD(NULL);
 	}
 	if (type->eType == eSimple && !strict_simple_type) {
 		t = choose_random_simple();
 		ERROR_GUARD(NULL);
-	} 
+	}
 	if (t->eType == eSimple) {
 		assert(t->simple_type != eVoid);
 	}
@@ -608,7 +653,7 @@ Type::choose_random_nonvoid_simple(void)
 #if 0
 	vector<unsigned int> vs;
 	vs.push_back(eVoid);
-	
+
 	if (!CGOptions::allow_int64()) {
 		vs.push_back(eLongLong);
 		vs.push_back(eULongLong);
@@ -623,7 +668,7 @@ Type::choose_random_nonvoid_simple(void)
 }
 
 void
-Type::make_one_bitfield(vector<const Type*> &random_fields, vector<CVQualifiers> &qualifiers, 
+Type::make_one_bitfield(vector<const Type*> &random_fields, vector<CVQualifiers> &qualifiers,
 			vector<int> &fields_length)
 {
 	int max_length = CGOptions::int_size() * 8;
@@ -635,7 +680,7 @@ Type::make_one_bitfield(vector<const Type*> &random_fields, vector<CVQualifiers>
 	CVQualifiers qual = CVQualifiers::random_qualifiers(type, FieldConstProb, FieldVolatileProb);
 	ERROR_RETURN();
 	qualifiers.push_back(qual);
-	int length = rnd_upto(max_length); 
+	int length = rnd_upto(max_length);
 	ERROR_RETURN();
 
 	bool no_zero_len = fields_length.empty() || (fields_length.back() == 0);
@@ -643,21 +688,22 @@ Type::make_one_bitfield(vector<const Type*> &random_fields, vector<CVQualifiers>
 	if (length == 0 && no_zero_len) {
 		if (max_length <= 2) length = 1;
 		else length = rnd_upto(max_length - 1) + 1;
-	} 
-	ERROR_RETURN(); 
+	}
+	ERROR_RETURN();
 	fields_length.push_back(length);
 }
 
 // ---------------------------------------------------------------------
 void
-Type::make_full_bitfields_struct_fields(size_t field_cnt, vector<const Type*> &random_fields, 
+Type::make_full_bitfields_struct_fields(size_t field_cnt, vector<const Type*> &random_fields,
 					vector<CVQualifiers> &qualifiers,
-					vector<int> &fields_length)
+					vector<int> &fields_length, 
+					bool structHasAssignOps)
 {
 	for (size_t i=0; i<field_cnt; i++) {
 		bool is_non_bitfield = rnd_flipcoin(ScalarFieldInFullBitFieldsProb);
 		if (is_non_bitfield) {
-			make_one_struct_field(random_fields, qualifiers, fields_length);
+			make_one_struct_field(random_fields, qualifiers, fields_length, structHasAssignOps);
 		}
 		else {
 			make_one_bitfield(random_fields, qualifiers, fields_length);
@@ -666,11 +712,12 @@ Type::make_full_bitfields_struct_fields(size_t field_cnt, vector<const Type*> &r
 }
 
 void
-Type::make_one_struct_field(vector<const Type*> &random_fields, 
+Type::make_one_struct_field(vector<const Type*> &random_fields,
 					vector<CVQualifiers> &qualifiers,
-					vector<int> &fields_length)
+					vector<int> &fields_length, 
+					bool structHasAssignOps)
 {
-	ChooseRandomTypeFilter f(/*for_field_var*/true);
+	ChooseRandomTypeFilter f(/*for_field_var*/true, structHasAssignOps);
 	unsigned int i = rnd_upto(AllTypes.size(), &f);
 	ERROR_RETURN();
 	const Type* type = AllTypes[i];
@@ -687,29 +734,36 @@ Type::make_one_union_field(vector<const Type*> &fields, vector<CVQualifiers> &qf
 	bool is_bitfield = CGOptions::bitfields() && !CGOptions::ccomp() && rnd_flipcoin(BitFieldInNormalStructProb);
 	if (is_bitfield) {
 		make_one_bitfield(fields, qfers, lens);
-	} 
+	}
 	else {
 		size_t i;
-		vector<Type*> ok_types;
-		// filter out struct types containing bit-fields. Their layout is implementation 
-		// defined, we don't want to mess with them in unions for now
-		for (i=0; i<AllTypes.size(); i++) {
-			if (!AllTypes[i]->has_bitfields()) {
-				ok_types.push_back(AllTypes[i]);
-			}
-		}
+			vector<Type*> ok_nonstruct_types;
+			vector<Type*> struct_types;
+			for (i = 0; i < AllTypes.size(); i++) {
+				if ((AllTypes[i]->eType != eStruct) && (AllTypes[i]->eType != eUnion)) {
+					ok_nonstruct_types.push_back(AllTypes[i]);
+					continue;
+				}
+				// filter out structs and unions containing bit-fields. Their layout is implementation 
+				// defined, we don't want to mess with them in unions for now
+				if (AllTypes[i]->has_bitfields())
+					continue;
 
-		// find of struct types 
-		vector<Type*> struct_types;
-		for (size_t i=0; i<ok_types.size(); i++) {
-			if (ok_types[i]->eType == eStruct) {
-				struct_types.push_back(ok_types[i]);
+				// filter out structs/unions with assign operators (C++ only) 
+				if (AllTypes[i]->has_implicit_nontrivial_assign_ops()) 
+					continue;
+
+				if (AllTypes[i]->eType == eStruct){
+					struct_types.push_back(AllTypes[i]);
+				}
+				else{	// union
+						// no union in union currently
+				}
 			}
-		}
 		const Type* type = NULL;
 		do {
-			// 10% chance to be struct field
-			if (!struct_types.empty() && pure_rnd_flipcoin(10)) { 
+			// 15% chance to be struct field
+			if (!struct_types.empty() && pure_rnd_flipcoin(15)) {
 				type = struct_types[pure_rnd_upto(struct_types.size())];
 				assert(type->eType == eStruct);
 			}
@@ -718,17 +772,15 @@ Type::make_one_union_field(vector<const Type*> &fields, vector<CVQualifiers> &qf
 				type = find_pointer_type(&get_simple_type(eChar), true);
 			}
 			else {
-				unsigned int i = pure_rnd_upto(ok_types.size()); 
-				const Type* t = ok_types[i];
-				// no union in union?
-				if (t->eType == eUnion || 
-					(t->eType == eSimple && SIMPLE_TYPES_PROB_FILTER->filter(t->simple_type))) {
+					unsigned int i = pure_rnd_upto(ok_nonstruct_types.size());
+					const Type* t = ok_nonstruct_types[i];
+				if (t->eType == eSimple && SIMPLE_TYPES_PROB_FILTER->filter(t->simple_type)) {
 					continue;
 				}
-				type = t; 
+				type = t;
 			}
-		} while (type == NULL); 
-	
+		} while (type == NULL);
+
 		fields.push_back(type);
 		CVQualifiers qual = CVQualifiers::random_qualifiers(type, FieldConstProb, FieldVolatileProb);
 		ERROR_RETURN();
@@ -738,9 +790,10 @@ Type::make_one_union_field(vector<const Type*> &fields, vector<CVQualifiers> &qf
 }
 
 void
-Type::make_normal_struct_fields(size_t field_cnt, vector<const Type*> &random_fields, 
+Type::make_normal_struct_fields(size_t field_cnt, vector<const Type*> &random_fields,
 					vector<CVQualifiers> &qualifiers,
-					vector<int> &fields_length)
+					vector<int> &fields_length, 
+					bool structHasAssignOps)
 {
 	for (size_t i=0; i<field_cnt; i++)
 	{
@@ -749,10 +802,10 @@ Type::make_normal_struct_fields(size_t field_cnt, vector<const Type*> &random_fi
 			make_one_bitfield(random_fields, qualifiers, fields_length);
 		}
 		else {
-			make_one_struct_field(random_fields, qualifiers, fields_length);
+			make_one_struct_field(random_fields, qualifiers, fields_length, structHasAssignOps);
 		}
 	}
-} 
+}
 
 #define ZERO_BITFIELD 0
 #define RANDOM_BITFIELD 1
@@ -777,16 +830,16 @@ Type::init_is_bitfield_enumerator(Enumerator<string> &enumerator, int bitfield_p
 }
 
 void
-Type::init_fields_enumerator(Enumerator<string> &enumerator, 
-				Enumerator<string> &bitfield_enumerator, 
-				int type_bound, int qual_bound, 
+Type::init_fields_enumerator(Enumerator<string> &enumerator,
+				Enumerator<string> &bitfield_enumerator,
+				int type_bound, int qual_bound,
 				int bitfield_qual_bound)
 {
 	int field_cnt = CGOptions::max_struct_fields();
 	for (int i = 0; i < field_cnt; ++i) {
 		std::ostringstream ss;
 		ss << "bitfield" << i;
-		bool is_bitfield = bitfield_enumerator.get_elem(ss.str());
+		bool is_bitfield = bitfield_enumerator.get_elem(ss.str()) != 0;
 		if (is_bitfield) {
 			std::ostringstream ss1, ss2, ss3;
 			ss1 << "bitfield_sign" << i;
@@ -846,7 +899,7 @@ Type::make_one_bitfield_by_enum(Enumerator<string> &enumerator,
 	ss2 << "bitfield_qualifier" << index;
 	ss3 << "bitfield_length" << index;
 
-	bool sign = enumerator.get_elem(ss1.str());
+	bool sign = enumerator.get_elem(ss1.str()) != 0;
 	// we cannot allow too many structs,
 	// so randomly choose the sign of fields.
 	if (pure_rnd_flipcoin(50))
@@ -859,7 +912,7 @@ Type::make_one_bitfield_by_enum(Enumerator<string> &enumerator,
 	qualifiers.push_back(qual);
 
 	int length_flag = enumerator.get_elem(ss3.str());
-	
+
 	int length = get_bitfield_length(length_flag);
 	if ((index==0 || last_is_zero) && (length == 0)) {
 		return false;
@@ -869,7 +922,7 @@ Type::make_one_bitfield_by_enum(Enumerator<string> &enumerator,
 	return true;
 }
 
-bool 
+bool
 Type::make_one_normal_field_by_enum(Enumerator<string> &enumerator, vector<const Type*> &all_types,
 				vector<CVQualifiers> &all_quals, vector<const Type*> &fields,
 				vector<CVQualifiers> &quals, vector<int> &fields_length, int i)
@@ -904,13 +957,13 @@ Type::make_one_normal_field_by_enum(Enumerator<string> &enumerator, vector<const
 
 void
 Type::make_all_struct_types_(Enumerator<string> &bitfields_enumerator, vector<const Type*> &accum_types,
-				vector<const Type*> &all_types, vector<CVQualifiers> &all_quals, 
+				vector<const Type*> &all_types, vector<CVQualifiers> &all_quals,
 				vector<CVQualifiers> &all_bitfield_quals)
 {
 	Enumerator<string> fields_enumerator;
-	init_fields_enumerator(fields_enumerator, bitfields_enumerator, all_types.size(), 
+	init_fields_enumerator(fields_enumerator, bitfields_enumerator, all_types.size(),
 			all_quals.size(), all_bitfield_quals.size());
-	
+
 	Enumerator<string> *i;
 	for (i = fields_enumerator.begin(); i != fields_enumerator.end(); i = i->next()) {
 		make_all_struct_types_with_bitfields(*i, bitfields_enumerator, accum_types, all_types, all_quals, all_bitfield_quals);
@@ -918,9 +971,9 @@ Type::make_all_struct_types_(Enumerator<string> &bitfields_enumerator, vector<co
 }
 
 void
-Type::make_all_struct_types_with_bitfields(Enumerator<string> &enumerator, 
+Type::make_all_struct_types_with_bitfields(Enumerator<string> &enumerator,
 				Enumerator<string> &bitfields_enumerator, vector<const Type*> &accum_types,
-				vector<const Type*> &all_types, vector<CVQualifiers> &all_quals, 
+				vector<const Type*> &all_types, vector<CVQualifiers> &all_quals,
 				vector<CVQualifiers> &all_bitfield_quals)
 {
 	vector<const Type*> fields;
@@ -934,7 +987,7 @@ Type::make_all_struct_types_with_bitfields(Enumerator<string> &enumerator,
 	for (int i = 0; i < field_cnt; ++i) {
 		std::ostringstream ss;
 		ss << "bitfield" << i;
-		bool is_bitfield = bitfields_enumerator.get_elem(ss.str());
+		bool is_bitfield = bitfields_enumerator.get_elem(ss.str()) != 0;
 		bool rv = false;
 		if (is_bitfield) {
 			rv = make_one_bitfield_by_enum(enumerator, all_bitfield_quals, fields, quals, fields_length, i, last_is_zero);
@@ -948,12 +1001,14 @@ Type::make_all_struct_types_with_bitfields(Enumerator<string> &enumerator,
 		if (!rv)
 			return;
 	}
-	if ((ExhaustiveBitFieldsProb > 0) && (ExhaustiveBitFieldsProb < 100) && 
+	if ((ExhaustiveBitFieldsProb > 0) && (ExhaustiveBitFieldsProb < 100) &&
 		((bitfields_cnt == field_cnt) || (normal_fields_cnt == field_cnt)))
 		return;
 
-	bool packed = enumerator.get_elem("packed");
-	Type* new_type = new Type(fields, true, packed, quals, fields_length);
+	bool packed = enumerator.get_elem("packed") != 0;
+	bool hasAssignOps = if_struct_will_have_assign_ops();
+	bool hasImplicitNontrivialAssignOps = hasAssignOps || checkImplicitNontrivialAssignOps(fields);
+    Type* new_type = new Type(fields, true, packed, quals, fields_length, hasAssignOps, hasImplicitNontrivialAssignOps);
 	new_type->used = true;
 	accum_types.push_back(new_type);
 }
@@ -1057,10 +1112,10 @@ Type::has_longlong_field(const vector<const Type *> &fields)
 
 Type*
 Type::make_random_struct_type(void)
-{ 
+{
     size_t field_cnt = 0;
     size_t max_cnt = CGOptions::max_struct_fields();
-    if (CGOptions::fixed_struct_fields()) 
+    if (CGOptions::fixed_struct_fields())
         field_cnt = max_cnt;
     else
         field_cnt = rnd_upto(max_cnt) + 1;
@@ -1070,12 +1125,12 @@ Type::make_random_struct_type(void)
     vector<int> fields_length;
     bool is_bitfields = CGOptions::bitfields() && rnd_flipcoin(BitFieldsCreationProb);
     ERROR_GUARD(NULL);
-
+	bool hasAssignOps = if_struct_will_have_assign_ops();
     //if (CGOptions::bitfields())
     if (is_bitfields)
-        make_full_bitfields_struct_fields(field_cnt, random_fields, qualifiers, fields_length);
+        make_full_bitfields_struct_fields(field_cnt, random_fields, qualifiers, fields_length, hasAssignOps);
     else
-        make_normal_struct_fields(field_cnt, random_fields, qualifiers, fields_length);
+        make_normal_struct_fields(field_cnt, random_fields, qualifiers, fields_length, hasAssignOps);
 
     ERROR_GUARD(NULL);
 
@@ -1090,34 +1145,36 @@ Type::make_random_struct_type(void)
             ERROR_GUARD(NULL);
         }
     }
-
-    Type* new_type = new Type(random_fields, true, packed, qualifiers, fields_length);
+    bool hasImplicitNontrivialAssignOps = hasAssignOps || checkImplicitNontrivialAssignOps(random_fields);
+    Type* new_type = new Type(random_fields, true, packed, qualifiers, fields_length, hasAssignOps, hasImplicitNontrivialAssignOps);
     return new_type;
 }
 
 Type*
 Type::make_random_union_type(void)
-{ 
-    size_t max_cnt = CGOptions::max_union_fields(); 
-    size_t field_cnt = rnd_upto(max_cnt) + 1;
-    ERROR_GUARD(NULL);
+{
+	size_t max_cnt = CGOptions::max_union_fields();
+	size_t field_cnt = rnd_upto(max_cnt) + 1;
+	ERROR_GUARD(NULL);
 
-    vector<const Type*> fields;
-    vector<CVQualifiers> qfers;
-    vector<int> lens;
-    
-	for (size_t i=0; i<field_cnt; i++) {
+	vector<const Type*> fields;
+	vector<CVQualifiers> qfers;
+	vector<int> lens;
+
+	for (size_t i = 0; i < field_cnt; i++) {
 		make_one_union_field(fields, qfers, lens);
 		assert(!fields.back()->has_bitfields());
-	} 
-    Type* new_type = new Type(fields, false, false, qfers, lens);
-    return new_type;
+	}
+	bool hasAssignOps = if_union_will_have_assign_ops();
+	bool hasImplicitNontrivialAssignOps = hasAssignOps || checkImplicitNontrivialAssignOps(fields);
+	Type* new_type = new Type(fields, false, false, qfers, lens, hasAssignOps, hasImplicitNontrivialAssignOps);
+	return new_type;
 }
 
 // ---------------------------------------------------------------------
 Type*
 Type::make_random_pointer_type(void)
-{ 
+{
     //Type* new_type = 0;
     //Type* ptr_type = 0;
     // occasionally choose pointer to pointers
@@ -1132,8 +1189,8 @@ Type::make_random_pointer_type(void)
 			}
 		}
 	}
-    
-    // choose a pointer to basic/aggregate types 
+
+    // choose a pointer to basic/aggregate types
 	const Type* t = choose_random();
 	ERROR_GUARD(NULL);
 	// consolidate all integer pointer types into "int*", hopefully this increase
@@ -1148,10 +1205,10 @@ Type::make_random_pointer_type(void)
 // ---------------------------------------------------------------------
 void
 Type::GenerateSimpleTypes(void)
-{ 
+{
     unsigned int st;
     for (st=eChar; st<MAX_SIMPLE_TYPES; st++)
-    { 
+    {
 		AllTypes.push_back(new Type((enum eSimpleType)st));
     }
     Type::void_type = new Type((enum eSimpleType)eVoid);
@@ -1161,7 +1218,7 @@ Type::GenerateSimpleTypes(void)
 void
 GenerateAllTypes(void)
 {
-	// In the exhaustive mode, we want to generate all type first. 
+	// In the exhaustive mode, we want to generate all type first.
 	// We don't support struct for now
 	if (CGOptions::dfs_exhaustive()) {
 		Type::GenerateSimpleTypes();
@@ -1172,14 +1229,14 @@ GenerateAllTypes(void)
 
 	Type::GenerateSimpleTypes();
     if (CGOptions::use_struct()) {
-        while (MoreTypesProbability()) { 
-		    Type *ty = Type::make_random_struct_type(); 
+        while (MoreTypesProbability()) {
+		    Type *ty = Type::make_random_struct_type();
 		    AllTypes.push_back(ty);
 	    }
     }
 	if (CGOptions::use_union()) {
-        while (MoreTypesProbability()) { 
-		    Type *ty = Type::make_random_union_type(); 
+        while (MoreTypesProbability()) {
+		    Type *ty = Type::make_random_union_type();
 		    AllTypes.push_back(ty);
 	    }
     }
@@ -1238,11 +1295,11 @@ Type::choose_random_simple(void)
 }
 
 // ---------------------------------------------------------------------
-int 
+int
 Type::get_indirect_level() const
 {
     int level = 0;
-    const Type* pt = ptr_type; 
+    const Type* pt = ptr_type;
     while (pt != 0) {
         level++;
         pt = pt->ptr_type;
@@ -1251,10 +1308,10 @@ Type::get_indirect_level() const
 }
 
 // ---------------------------------------------------------------------
-int 
+int
 Type::get_struct_depth() const
 {
-    int depth = 0; 
+    int depth = 0;
     if (eType == eStruct) {
 		depth++;
 		int max_depth = 0;
@@ -1290,7 +1347,7 @@ Type::is_bitfield(size_t index) const
 bool
 Type::has_bitfields() const
 {
-	for (size_t i=0; i<fields.size(); i++) { 
+	for (size_t i=0; i<fields.size(); i++) {
 		if (bitfields_length_[i] >= 0) {
 			return true;
 		}
@@ -1301,9 +1358,9 @@ Type::has_bitfields() const
 	return false;
 }
 
-// conservatively assume padding is present in all unpacked structures 
+// conservatively assume padding is present in all unpacked structures
 // or whenever there is bitfields
-bool 
+bool
 Type::has_padding(void) const
 {
 	if (eType == eStruct && !packed_) return true;
@@ -1333,7 +1390,7 @@ Type::is_signed(void) const
 	switch (eType) {
 	default:
 		return false;
-		
+
 	case eSimple:
 		switch (simple_type) {
 		case eUChar:
@@ -1374,7 +1431,7 @@ Type::to_unsigned(void) const
 	return NULL;
 }
 
-const Type* 
+const Type*
 Type::get_base_type(void) const
 {
 	const Type* tmp = this;
@@ -1390,12 +1447,12 @@ Type::is_promotable(const Type* t) const
     if (eType == eSimple && t->eType == eSimple) {
 		eSimpleType t2 = t->simple_type;
 		switch (simple_type) {
-			case eChar:  
-			case eUChar: return (t2 != eVoid); 
-			case eShort: 
-			case eUShort: return (t2 != eVoid && t2 != eChar && t2 != eUChar); 
+			case eChar:
+			case eUChar: return (t2 != eVoid);
+			case eShort:
+			case eUShort: return (t2 != eVoid && t2 != eChar && t2 != eUChar);
 			case eInt:
-			case eUInt: return (t2 != eVoid && t2 != eChar && t2 != eUChar && t2 != eShort && t2 != eUShort); 
+			case eUInt: return (t2 != eVoid && t2 != eChar && t2 != eUChar && t2 != eShort && t2 != eUShort);
 			case eLong:
 			case eULong: return (t2 == eLong || t2 == eULong || t2 == eLongLong || t2 == eULongLong);
 			case eLongLong:
@@ -1410,19 +1467,19 @@ Type::is_promotable(const Type* t) const
 // ---------------------------------------------------------------------
 /* generally integer types can be converted to any other interger types
  * void / struct / union / array types are not. Pointers depend on the
- * type they point to. (unsigned int*) is convertable to (int*), etc  
+ * type they point to. (unsigned int*) is convertable to (int*), etc
  *************************************************************/
 bool
 Type::is_convertable(const Type* t) const
 {
-    if (this == t) 
+    if (this == t)
         return true;
     if (eType == eSimple && t->eType == eSimple) {
 	// forbiden conversion from float to int
 	if (t->is_float() && !is_float())
 		return false;
         if ((simple_type != eVoid && t->simple_type != eVoid) ||
-            simple_type == t->simple_type) 
+            simple_type == t->simple_type)
             return true;
     }
     else if (eType == ePointer && t->eType == ePointer) {
@@ -1430,8 +1487,14 @@ Type::is_convertable(const Type* t) const
 		return true;
 	}
 	if (ptr_type->eType == eSimple && t->ptr_type->eType == eSimple) {
-                if(ptr_type->simple_type == eFloat && t->ptr_type->simple_type == eFloat)
-                    return true;
+		if (ptr_type->simple_type == t->ptr_type->simple_type)
+			return true;
+		else if (CGOptions::strict_float() &&
+			((ptr_type->simple_type == eFloat && t->ptr_type->simple_type != eFloat) ||
+                	 (ptr_type->simple_type != eFloat && t->ptr_type->simple_type == eFloat)))
+		    return false;
+		else if (CGOptions::lang_cpp())
+			return false;	// or we need an explicit cast here
 		else
 		    return ptr_type->SizeInBytes() == t->ptr_type->SizeInBytes();
 	}
@@ -1461,12 +1524,12 @@ Type::needs_cast(const Type* t) const
 	return (eType == ePointer) && !get_base_type()->is_equivalent(t->get_base_type());
 }
 
-bool 
+bool
 Type::match(const Type* t, enum eMatchType mt) const
 {
 	switch (mt) {
 	case eExact: return (this == t);
-	case eConvert: return is_convertable(t); 
+	case eConvert: return is_convertable(t);
 	case eDereference: return is_dereferenced_from(t);
 	case eDerefExact: return (t == this || is_dereferenced_from(t));
 	case eFlexible: return is_derivable(t);
@@ -1531,7 +1594,7 @@ Type::SizeInBytes(void) const
 //		case eDouble:	return 8;
 		}
 		break;
-	case eUnion: { 
+	case eUnion: {
         unsigned int max_size = 0;
         for (i=0; i<fields.size(); i++) {
 			unsigned int sz = 0;
@@ -1548,12 +1611,12 @@ Type::SizeInBytes(void) const
         }
         return max_size;
     }
-    case eStruct: { 
+    case eStruct: {
 		if (!this->packed_) return SIZE_UNKNOWN;
 		// give up if there are bitfields, too much compiler-dependence and machine-dependence
 		if (this->has_bitfields()) return SIZE_UNKNOWN;
         unsigned int total_size = 0;
-        for (i=0; i<fields.size(); i++) { 
+        for (i=0; i<fields.size(); i++) {
 			unsigned int sz = fields[i]->SizeInBytes();
 			if (sz == SIZE_UNKNOWN) return sz;
             total_size += sz;
@@ -1580,15 +1643,15 @@ Type::SelectLType(bool no_volatile, eAssignOps op)
 	// (see Statement::make_random)
 	if (op == eSimpleAssign && rnd_flipcoin(PointerAsLTypeProb)) {
 		ERROR_GUARD(NULL);
-		type = Type::make_random_pointer_type(); 
+		type = Type::make_random_pointer_type();
 	}
 	ERROR_GUARD(NULL);
 
 	// choose a struct type as LHS type
-	if (!type) {
+	if (!type && (op == eSimpleAssign)) {
 		vector<Type *> ok_struct_types;
 		get_all_ok_struct_union_types(ok_struct_types, true, no_volatile, false, true);
-		if ((ok_struct_types.size() > 0) && (op == eSimpleAssign) && rnd_flipcoin(StructAsLTypeProb)) {
+		if ((ok_struct_types.size() > 0)  && rnd_flipcoin(StructAsLTypeProb)) {
 			type = Type::choose_random_struct_union_type(ok_struct_types);
 		}
 	}
@@ -1607,17 +1670,19 @@ Type::SelectLType(bool no_volatile, eAssignOps op)
 	return type;
 }
 
-void 
-Type::get_int_subfield_names(string prefix, vector<string>& names, const vector<int>& excluded_fields) const
+void
+Type::get_int_subfield_names(string prefix, vector<string>& names,
+		vector<const Type *>& types, const vector<int>& excluded_fields) const
 {
 	if (eType == eSimple) {
 		names.push_back(prefix);
+		types.push_back(this);
 	}
 	else if (is_aggregate()) {
 		size_t i;
 		size_t j = 0;
 		for (i=0; i<fields.size(); i++) {
-			if (is_unamed_padding(i)) continue; // skip 0 length bitfields 
+			if (is_unamed_padding(i)) continue; // skip 0 length bitfields
 			// skip excluded fields
 			if (std::find(excluded_fields.begin(), excluded_fields.end(), j) != excluded_fields.end()) {
 				j++;
@@ -1626,12 +1691,12 @@ Type::get_int_subfield_names(string prefix, vector<string>& names, const vector<
 			ostringstream oss;
 			oss << prefix << ".f" << j++;
 			vector<int> empty;
-			fields[i]->get_int_subfield_names(oss.str(), names, empty);
+			fields[i]->get_int_subfield_names(oss.str(), names, types, empty);
 		}
 	}
 }
 
-bool 
+bool
 Type::contain_pointer_field(void) const
 {
 	if (eType == ePointer) return true;
@@ -1659,7 +1724,7 @@ Type::Output(std::ostream &out) const
 			out << (is_signed() ? "int" : "uint");
 			out << (SizeInBytes() * 8);
 			out << "_t";
-		} 
+		}
 		break;
 	case ePointer:   ptr_type->Output( out ); out << "*"; break;
 	case eUnion:     out << "union U" << sid; break;
@@ -1677,6 +1742,107 @@ Type::get_type_sizeof_string(std::string &s) const
 	s = ss.str();
 }
 
+// If this is C++, create assign operators for volatile structs:
+// Semantically this should work, though for bit fields it's probably not what was intended.
+//volatile struct S0& operator=(const volatile struct S0& val) volatile {
+//	if (this == &val) {
+//		return *this;
+//	}
+//	f0 = val.f0;
+//	f1 = val.f1;
+//	return *this;
+//}
+// As a result of generating this, have to generate 'default' assignment operator as well
+void OutputStructAssignOp(Type* type, std::ostream &out, bool vol)
+{
+	if (CGOptions::lang_cpp()){
+		if (type->has_assign_ops() && (type->eType == eStruct)){
+			out << "    "; 
+			if (vol){
+				out << "volatile ";
+			}
+			type->Output(out); out << "& operator=(const ";
+			if (vol){
+				out << "volatile ";
+			}
+			type->Output(out); out << "& val) ";
+			if (vol){
+				out << "volatile ";
+			}
+			out << "{"; really_outputln(out);
+			out << "        if (this == &val) {"; really_outputln(out);
+			out << "            return *this;"; really_outputln(out);
+			out << "        }"; really_outputln(out);
+			// for all fields:
+			for (size_t i = 0, j = 0; i<type->fields.size(); i++) {
+				int length = type->bitfields_length_[i];
+				if (length != 0){
+					out << "        ";
+					out << " f" << j << "= val.f" << j << ";";
+					really_outputln(out);
+					++j;
+				}
+			}
+
+			out << "        return *this;"; really_outputln(out);
+			out << "    }"; really_outputln(out);
+		}
+	}
+}
+
+// To have volatile unions in C++ they need assignment operators:
+//union U1& operator=(const union U1& val){
+//	if (this == &val) {
+//		return *this;
+//	}
+//	memcpy(this, &val, sizeof(union U1));
+//	return *this;
+//}
+//volatile union U1& operator=(const volatile union U1& val) volatile {
+//	if (this == &val) {
+//		return *this;
+//	}
+//	memcpy((union U1*)this, (const union U1*)(&val), sizeof(union U1));
+//	return *this;
+//}
+
+void OutputUnionAssignOps(Type* type, std::ostream &out, bool vol)
+{
+	if (CGOptions::lang_cpp()){
+		if (type->has_assign_ops() && (type->eType == eUnion)){
+
+			out << "    ";
+			if (vol){
+				out << "volatile ";
+			}
+			type->Output(out); out << "& operator=(const ";
+			if (vol){
+				out << "volatile ";
+			}
+			type->Output(out); out << "& val) ";
+			if (vol){
+				out << "volatile ";
+			}
+			out << "{"; really_outputln(out);
+			out << "        if (this == &val) {"; really_outputln(out);
+			out << "            return *this;"; really_outputln(out);
+			out << "        }"; really_outputln(out);
+
+
+			out << "        memcpy((";
+			type->Output(out);
+			out << "*)this, (const ";
+			type->Output(out);
+			out << "*)(&val), sizeof(";
+			type->Output(out);
+			out << ")); "; really_outputln(out);
+
+			out << "        return *this;"; really_outputln(out);
+			out << "    }"; really_outputln(out);
+		}
+	}
+}
+
 // ---------------------------------------------------------------------
 /* print struct definition (fields etc)
  *************************************************************/
@@ -1686,13 +1852,13 @@ void OutputStructUnion(Type* type, std::ostream &out)
     // sanity check
     assert (type->is_aggregate());
 
-    if (!type->printed) { 
+    if (!type->printed) {
         // output dependent structs, if any
-        for (i=0; i<type->fields.size(); i++) { 
+        for (i=0; i<type->fields.size(); i++) {
 			if (type->fields[i]->is_aggregate()) {
                 OutputStructUnion((Type*)type->fields[i], out);
             }
-        } 
+        }
         // output myself
         if (type->packed_) {
             if (!CGOptions::ccomp()) {
@@ -1706,7 +1872,7 @@ void OutputStructUnion(Type* type, std::ostream &out)
         out << " {";
 		really_outputln(out);
 
-		assert(type->fields.size() == type->qfers_.size()); 
+		assert(type->fields.size() == type->qfers_.size());
 		unsigned int j = 0;
         for (i=0; i<type->fields.size(); i++) {
             out << "   ";
@@ -1735,6 +1901,16 @@ void OutputStructUnion(Type* type, std::ostream &out)
 			}
 			really_outputln(out);
         }
+
+		if (type->eType == eStruct){
+			OutputStructAssignOp(type, out, false);
+			OutputStructAssignOp(type, out, true);
+		}
+		else{
+			OutputUnionAssignOps(type, out, false);
+			OutputUnionAssignOps(type, out, true);
+		}
+
         out << "};";
 		really_outputln(out);
         if (type->packed_) {
@@ -1771,7 +1947,7 @@ OutputStructUnionDeclarations(std::ostream &out)
 /*
  * return the printf directive string for the type. for example, int -> "%d"
  */
-std::string 
+std::string
 Type::printf_directive(void) const
 {
 	string ret;
@@ -1784,17 +1960,17 @@ Type::printf_directive(void) const
 			ret = is_signed() ? "%d" : "%u";
 		}
 		break;
-	case ePointer:   
-		ret = "0x%0x"; 
+	case ePointer:
+		ret = "0x%0x";
 		break;
-	case eUnion:   
-	case eStruct:  
+	case eUnion:
+	case eStruct:
 		ret = "{";
 		for (i=0; i<fields.size(); i++) {
 			if (i > 0) ret += ", ";
 			ret += fields[i]->printf_directive();
 		}
-		ret += "}"; 
+		ret += "}";
 		break;
 	}
 	return ret;
